#!/bin/sh

LOCK_FILE=/var/lock/LCK..pkgmgrd.lock
PM_LOGFILE=/var/tmp/pkgmgrd.log

if [ "x$SBOX_PRELOAD" != "x" ]; then
    LD_PRELOAD=$SBOX_PRELOAD
    FAKEROOT=fakeroot
else
    unset AEGIS_FIXED_ORIGIN
fi

SCRIPT_DIR=$(cd `dirname $0`; pwd)
DAEMON=$SCRIPT_DIR/../daemon/pkgmgrd
if [ ! -x $DAEMON ]; then
    PATH=/usr/share/package-manager-testrepo:$PATH
    DAEMON=/usr/bin/pkgmgrd
else
    PATH=$SCRIPT_DIR/testrepo:$PATH
fi

# Kill pkgmgrd if already running
if test -f $LOCK_FILE; then
    pid=`cat $LOCK_FILE`
    if test "$pid" -gt 1; then
	# SIGCONT
	kill -18 $pid
	if [ $? -ne 0 ]; then
	    echo "$0: $LOCK_FILE with stale pid $pid, removed"
	    rm $LOCK_FILE
	else
	    echo "pkgmgrd[$pid] already running"
	    exit 1 # Fail if it can not be killed :(
	fi
    fi
fi

alarm() {
    perl -e 'alarm shift; exec @ARGV' "$@";
}

ensure-meego-sb-session

start_ts=`date +%s`
#echo "$FAKEROOT $DAEMON -v -l $PM_LOGFILE >/dev/null 2>&1 &"
alarm 130 $FAKEROOT $DAEMON -v -l $PM_LOGFILE >/dev/null

if [ $? -ne 0 ]; then
    echo "Failed. Pkgmgrd idled too long"
    exit 1
fi

end_ts=`date +%s`
result=$(($end_ts-$start_ts))
echo "Result: $result s"

if test $result -lt 119; then
  echo "Failed. Pkgmgrd died too early"
  exit 1
fi

if test $result -gt 128; then
  echo "Failed. Pkgmgrd idled too long"
  exit 1
fi

echo "OK"
exit 0
