# -*- python -*-
#
# Copyright (C) 2009 - 2011 Nokia Corporation.
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License version
# 2 as published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see http://www.gnu.org/licenses/.

#
# TODO: Check exception strings?

import unittest
import dbus
import commands
import sys

from test_helper import *

# whether to check fetching lists fully or only our test repo packages
full_fetch_check = False

class tc_pkgmgrd_dbus_api_catalogue(tc_pkgmgrd_helper):
    def setUp(self):
        tc_pkgmgrd_helper.setUp(self)
        self.catalogue_name = 'TestCatalogue'
        self.catalogue_uri = 'file:///tmp/' + self.catalogue_name
        self.catalogue_dist = 'default'
        self.catalogue_comps = 'main user'

        self.iface.AddCatalogue(self.catalogue_name, self.catalogue_uri, self.catalogue_dist, self.catalogue_comps)
        self.failUnless(catalogue_exists(self.iface, self.catalogue_name))

    def tearDown(self):
        self.iface.RemoveCatalogue(self.catalogue_name)
        self.failIf(catalogue_exists(self.iface, self.catalogue_name))

        tc_pkgmgrd_helper.tearDown(self)

    def test_FetchCatalogues_all(self):
        """
        ApMgr-40: Review repository list
        @Fetch all catalogues (=repositories)
        """
        cats = self.iface.FetchCatalogues()
        results = []
        for cat in cats:
            results.append(cat['Name'])
        self.failUnless(self.catalogue_name in results, "Got: %s" %(results))

    def test_AddCatalogue_already_exists(self):
        self.assertRaises(dbus.DBusException, self.iface.AddCatalogue, self.catalogue_name, self.catalogue_uri, self.catalogue_dist, self.catalogue_comps)

    def test_AddCatalogue_name_already_exists(self):
        self.assertRaises(dbus.DBusException, self.iface.AddCatalogue, self.catalogue_name, 'file:///tmp/unique-uri', self.catalogue_dist, self.catalogue_comps)

    def test_AddCatalogue_info_already_exists(self):
        # Info (uri, dist, comps) already exists
        self.assertRaises(dbus.DBusException, self.iface.AddCatalogue, 'UniqueName', self.catalogue_uri, self.catalogue_dist, self.catalogue_comps)
        self.failIf(catalogue_exists(self.iface, 'UniqueName'))

    def test_EnableCatalogue_disable_enable(self):
        # True->False
        self.iface.EnableCatalogue(self.catalogue_name, False)
        # False->False
        self.iface.EnableCatalogue(self.catalogue_name, False)
        # False->True
        self.iface.EnableCatalogue(self.catalogue_name, True)
        # True->True
        self.iface.EnableCatalogue(self.catalogue_name, True)
