# -*- python -*-
#
# Copyright (C) 2009 - 2011 Nokia Corporation.
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License version
# 2 as published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see http://www.gnu.org/licenses/.

#
# TODO: Check exception strings?

import unittest
import dbus
import commands
import sys

from test_helper import *

# whether to check fetching lists fully or only our test repo packages
full_fetch_check = False

def pkg_filename(path, name, version='0.1.0-1', arch=deb_arch()):
    return '%s/%s_%s_%s.deb' % (path, name, version, arch)

def dbus_error(str):
    return 'com.nokia.package_manager.Error.'+str

class test_daemon_dbus_fetch(tc_pkgmgrd_helper):
    def setUp(self):
        tc_pkgmgrd_helper.setUp(self)
        setup_testrepo_upgrade_packages()
        pkgdir = get_pkgdir()
        arch = deb_arch()
        userdir = get_repodir()+'/pool/user/desktop'
        maindir = get_repodir()+'/pool/main/misc'
        self.pkgname = 'hpm-user1'
        self.syspkgname = 'hpm-system-meta'
        self.pkgversion = '0.1.0-1'
        self.syspkgversion = '0.1.2-1'
        self.pkgfilename = pkg_filename(userdir, self.pkgname)
        self.nonuserpkgname = 'hpm-nonuser'
        self.nonuserpkgfilename = pkg_filename(maindir, self.nonuserpkgname)
        self.visiblepkgname = 'hpm-user-visible'
        self.visiblepkgfilename = pkg_filename(maindir, self.visiblepkgname)
        self.invalidarchpkgname = 'hpm-user-invalid-arch'
        self.invalidarchpkgfilename = pkg_filename(userdir, self.invalidarchpkgname, self.pkgversion, 'powerpc')
        self.missingdepspkgname = 'hpm-user-missing-deps'
        self.missingdepspkgfilename = pkg_filename(userdir, self.missingdepspkgname)
        self.dependencypkgname = 'hpm-user-dep'
        self.dependencypkgfilename = pkg_filename(userdir, self.dependencypkgname)
        self.nonuserdependencypkgname = 'hpm-nonuser-dep'
        self.nonuserdependencypkgfilename = pkg_filename(userdir, self.nonuserdependencypkgname)
        self.validdepspkgname = 'hpm-user-valid-deps'
        self.validdepspkgfilename = pkg_filename(userdir, self.validdepspkgname)
        self.localpkgname = 'local-package'
        self.localpkgversion = '1.0'
        self.localpkgfilename = get_local_pkgdir() + '/local-package-1.0.deb'
	self.specialcharpkgname = 'local-specialchar-package'
	self.specialcharpkgfilename = get_local_pkgdir() + '/local-specialchar-package-1.0.deb'
	self.localpercentpkgname = 'local-package-percent'
        self.localpercentpkgversion = '1.0'
        self.localpercentpkgfilename = get_local_pkgdir() + '/local-package-%-1.0.deb'

    def tearDown(self):
        tc_pkgmgrd_helper.tearDown(self)
        
    def async_reply(self):
        self.loop.quit()

    def async_error(self, e):
        self.error = e
        self.loop.quit()

    def on_package_ignored(self, ign_type, pkg, ver):
        self.package_ignored = (ign_type, pkg, ver)
        self.loop.quit()

    def test_FetchInstalled_all(self):
        """
        ApMgr-27: Browse Installed
        @Fetch all installed packages
        """
        p = self.localpkgname
        v = self.localpkgversion
        f = self.localpkgfilename

        self._RefreshPackageCache("", "")
        self.check_operation_complete("Refresh", "")
        self.clear_signals()

        self.failUnless(os.path.isfile(f))
        self.check_package_not_installed(p)
        self.check_package_not_available(p)

#        print p
#        print f
#        print ''
        self._InstallFile(f)
        self.check_install(p, v, False, f)
        self.clear_signals()
        self.check_package_installed(p)
        
        name = 'hpm-user2'
        ver = '0.1.1-1'
        self._Install(name)
        self.check_install(name, ver)
        self.clear_signals()
        self.check_package_installed_or_available(name, ver, True)

        pkgs = self._FetchInstalled()
        installed = get_installed_packages_list()
        results = []
        cnt = 0
        for pkg in pkgs:
            n = str(pkg['Name'])
            self.assertEqual(pkg['Type'], 'Installed')
#            print n
            results.append((n, str(pkg["Version"])))
            self.package_has_timestamp( self.timestamp_storage_file, n, pkg['Version'] )

            if ( n != p and 'hpm-' in n):
#                print n
                self.assertEqual(pkg["Maemo-Display-Name-sv_SE"], "Fancy %s (sv_SE)" % n)
                self.assertEqual(pkg["Maemo-Display-Name-de_DE"], "Fancy %s (de_DE)" % n)
                self.assertEqual(pkg["Maemo-Display-Name-fi_FI"], "Fancy %s (fi_FI)" % n)
            
            if ( n == p or n == name ):
                try:
                    ts = pkg['InstalledTimestamp']
                except KeyError:
                    self.fail("Timestamp is missing!")
                if ( n == p ):
#                    print 'Pkg found: %s' % n
                    self.assertEqual(pkg['Filename'], f)
                    self.assertEqual(pkg['FlagLocalFile'], True)
                else:
                    self.assert_('FlagLocalFile' not in pkg)

        results.sort()
        installed.sort()
        self.failUnless(installed == results, "Installed: %s\nGot: %s" %(installed, results))
        
        self._Uninstall(p)
        self.check_uninstall(p)
        self.clear_signals()
        self.check_package_not_installed(p)
        
        self._Uninstall(name)
        self.check_uninstall(name)
        self.clear_signals()
        self.check_package_installed_or_available(name, '', False)


    def test_Fetch_localPackage_ok(self):
        p = self.localpkgname
        v = self.localpkgversion
        f = self.localpkgfilename

        self._RefreshPackageCache("", "")
        self.check_operation_complete("Refresh", "")
        self.clear_signals()

        self.failUnless(os.path.isfile(f))
        self.check_package_not_installed(p)
        self.check_package_not_available(p)

        res = self._FetchPackageDataFile(f)
#        for i in res:
#            print '%s:\t%s' % (i, res[i])
        self.assertEqual(res['Name'], p)
        self.assertEqual(res['Version'], v)
        self.assertEqual(res['Type'], 'NotInstalled')
        self.assertEqual(res['FlagLocalFile'], True)

        self._InstallFile(f)
        self.check_install(p, v, False, f)
        self.clear_signals()
        self.check_package_installed(p)

        res = self._FetchPackageDataFile(f)
#        for i in res:
#            print '%s:\t%s' % (i, res[i])
        self.assertEqual(res['Name'], p)
        self.assertEqual(res['Version'], v)
        self.assertEqual(res['Type'], 'Installed')
        self.assertEqual(res['FlagLocalFile'], True)

        # Uninstall the package
        self._Uninstall(p)
        self.check_uninstall(p)
        self.clear_signals()
        self.check_package_not_installed(p)
        
        res = self._FetchPackageDataFile(f)
#        for i in res:
#            print '%s:\t%s' % (i, res[i])
        self.assertEqual(res['Name'], p)
        self.assertEqual(res['Version'], v)
        self.assertEqual(res['Type'], 'NotInstalled')
        self.assertEqual(res['FlagLocalFile'], True)


    def test_Fetch_localPackage_percent_mark_in_its_filename(self):
        p = self.localpercentpkgname
        v = self.localpercentpkgversion
        f = self.localpercentpkgfilename

        self._RefreshPackageCache("", "")
        self.check_operation_complete("Refresh", "")
        self.clear_signals()

        self.failUnless(os.path.isfile(f))
        self.check_package_not_installed(p)
        self.check_package_not_available(p)

        res = self._FetchPackageDataFile(f)
#        for i in res:
#            print '%s:\t%s' % (i, res[i])
        self.assertEqual(res['Name'], p)
        self.assertEqual(res['Version'], v)
        self.assertEqual(res['Type'], 'NotInstalled')
        self.assertEqual(res['FlagLocalFile'], True)


    def test_FetchPackages_all(self):
        """
        ApMgr-??: ??
        @Fetch all available packages
        """
        pkgs = self._FetchPackages('')
        results = []
        for pkg in pkgs:
            results.append(pkg['Name'])
        if full_fetch_check:
            available = get_available_packages_list()
            available.sort()
            results.sort()
            self.failUnless(available == results, "Available: %s\nGot: %s" %(available, results))
        else:
            available = ["hpm-user1"]
            for pkg in available:
                self.failUnless(pkg in results)

    def test_FetchPackages_desktop(self):
        """
        ApMgr-??: ??
        @Fetch all available packages
        """
        pkgs = self._FetchPackages('Desktop')
        results = []
        for pkg in pkgs:
            results.append(pkg['Name'])

        if full_fetch_check:
            available = get_available_packages_list('Desktop')
            available.sort()
            results.sort()
            self.failUnless(available == results, "Available: %s\nGot: %s" %(available, results))
        else:
            available = ["hpm-user1"]
            for pkg in available:
                self.failUnless(pkg in results)

    def test_FetchPackageDataFile_ok(self):
        """
        ApMgr-32: Display Description
        @Fetch standalone package details
        """
        res = self._FetchPackageDataFile(self.pkgfilename)
        self.assertEqual(res["Name"], self.pkgname)
        self.assertEqual(res["Category"], "Desktop")
        self.assertEqual(res["Repository"], "local-file")
        self.assertEqual(res["Version"], self.pkgversion)
        self.assertEqual(res["Description"], "hpm-user1 package\nLong description for package hpm-user1")
        self.assertEqual(res["FlagLocalFile"], True)

    def test_FetchPackageDataInstalled_Localfile(self):
        """
        ApMgr-32: Display Description
        @Fetch standalone package details
        """
        p = 'local-package'
        v = '1.0'
        f = get_local_pkgdir() + '/local-package-1.0.deb'
        
        self._InstallFile(f)
        self.check_install(p, v, False, f)
        self.clear_signals()
        self.check_package_installed(p)
        
        res = self._FetchPackageDataInstalled(p)
        self.assertEqual(res["Name"], p)
        self.assertEqual(res["Version"], v)
        self.assertEqual(res["Type"], 'Installed')
        self.assertEqual(res["Filename"], f)
        self.assertEqual(res["Category"], "Other")
        self.assertEqual(res["Repository"], "local-file")
        self.assertEqual(res["Description"], "local test package\nThis package is for testing installation from a local file. It should not be in any repository.")
        self.assertEqual(res["FlagLocalFile"], True)
        
        # Uninstall the package
        self._Uninstall(p)
        self.check_uninstall(p)
        self.clear_signals()
        self.check_package_not_installed(p)

    def test_FetchPackageDataFile_visible(self):
        res = self._FetchPackageDataFile(self.visiblepkgfilename)
        self.assertEqual(res["Name"], self.visiblepkgname)
        self.assertEqual(res["Version"], self.pkgversion)
        self.assertEqual(res["FlagLocalFile"], True)
        
    def test_FetchPackageDataFile_notFound(self):
        """
        ApMgr-32: Display Description
        @Try to fetch non-existing standalone package details
        """
        self.failUnlessRaisesDBusException('FileNotFound', self._FetchPackageDataFile, 'non-existing-filename')
        self.failUnlessRaisesDBusException('FileNotFound', self._FetchPackageDataFile, '')

    def test_FetchPackageDataFile_invalidFile(self):
        """
        ApMgr-32: Display Description
        @Try to fetch standalone package details with only a pathname
        """
        self.failUnlessRaisesDBusException('InvalidFile', self._FetchPackageDataFile, '/')

    def test_FetchPackageDataFile_nonuser_pkg(self):
        """
        ApMgr-32: Display Description
        @Try to Fetch standalone package details from nonuser package
        """
        self.assertRaises(dbus.DBusException, self._FetchPackageDataFile, self.nonuserpkgfilename)

    def test_FetchPackageData_ok(self):
        """
        ApMgr-32: Display Description
        @Fetch repository package details
        """
        res = self._FetchPackageData(self.pkgname,self.pkgversion)
        self.assertEqual(res['Origin'], 'com.nokia.maemo/testrepo')
        self.assertEqual(res["Name"], self.pkgname)
        self.assertEqual(res["Type"], "NotInstalled")
        self.assertEqual(res["Category"], "Desktop")
        self.assertEqual(res["Version"], "0.1.0-1")
        self.assertEqual(res["Description"], "hpm-user1 package\nLong description for package hpm-user1")
        self.assert_('FlagLocalFile' not in res)
    
    def test_FetchPackageData_upgrade(self):
        """
        PkMgr-1173: Display Description
        @Fetch repository package details
        """
        name = 'hpm-user-valid-deps3'
        ver0 = '0.1.0-1'
        ver1 = '0.1.1-1'
        self.check_package_installed_or_available(name, '', False)
        D = ['hpm-user-dep2', 'hpm-user-dep4', 'hpm-user-dep5', 'hpm-user-dep6']
        for d in D:
            self.check_package_installed_or_available(d, '', False)

        self.downgrade_testrepo()
        
        self._RefreshPackageCache("", "")
        self.check_operation_complete("Refresh", "")
        self.clear_signals()

        clear_test_packages_in_apt_archives_dir()

        # Depends on hpm-user2 (= 0.1.0-1) which would be upgraded
        self._Install('hpm-user-old-deps', ver0)
        self.clear_signals()
        self.check_package_installed_version('hpm-user-dep2', ver0)

        self._Install(name, ver0)
        self.check_install(name, ver0, True, None, "", True, True)
        self.clear_signals()
        self.check_package_installed_or_available(name, ver0, True)

        setup_testrepo_upgrade_packages()
        
        self._RefreshPackageCache("", "")
        self.check_refresh_cache(True)
        self.clear_signals()

        P = self._FetchPackageData(name, ver1)

#        for p in P:
#            print "%s:\t%s" % (p, P[p])

        self.assertEqual(P["Name"], name)
        self.assertEqual(P["Category"], "Programming")
        self.assertEqual(P["Version"], ver1)
        self.assertEqual(P["Type"], "Update")
        self.assertEqual(P["Description"], "hpm-user-valid-deps3 package\nLong description for package hpm-user-valid-deps3")
        self.failUnless(P["DownloadSize"] > 0)
        # For an upgrade, this may well be <= 0
#        self.failUnless(P["TotalInstalledSize"] > 0)
        self.assertEqual(P["InstallDeps"], ["hpm-user-dep4=%s" % ver1, "hpm-user-dep5=%s" % ver1])
        self.assertEqual(P["UpgradeDeps"], ["hpm-user-dep2=%s" % ver1])
        self.assertEqual(P["RemovePkgs"], ["hpm-user-old-deps=%s" % ver0, "hpm-user-dep6=%s" % ver0])
        self.assert_('FlagLocalFile' not in P)

        self._Uninstall(name)
        self.check_uninstall(name, True)
        self.clear_signals()

        self._Uninstall('hpm-user-old-deps')
        self.check_uninstall('hpm-user-old-deps', True)
        self.clear_signals()

        for d in D:
            self.check_package_installed_or_available(d, '', False)

    def test_FetchSystemPackageData_ok(self):
        """
        ApMgr-32, PkMgr-1005: Display Description
        @Fetch repository system package details. Also check that the reboot flag is true.
        """
        res = self._FetchPackageData(self.syspkgname,self.syspkgversion)
#        print res
        self.assertEqual(res["Name"], self.syspkgname)
        self.assertEqual(res["Category"], "System")
        self.assertEqual(res["Version"], self.syspkgversion)
        self.assertEqual(res["Description"], "hpm-system-meta package\nLong description for package hpm-system-meta")
        self.assertEqual(res["FlagReboot"], True)
        self.assert_('FlagLocalFile' not in res)

    def test_FetchPackageData_invalid_pkg_latest(self):
        """
        ApMgr-32: Display Description
        @Try to fetch repository package details of latest version of nonexisting package
        """
        name='invalidpackagename'
        self.failUnlessRaisesDBusException('PackageNotFound', self._FetchPackageData, name, "")

    def test_FetchPackageData_invalid_pkg(self):
        """
        ApMgr-32: Display Description
        @Try to fetch repository package details of nonexisting package
        """
        name='invalidpackagename'
        self.failUnlessRaisesDBusException('PackageNotFound', self._FetchPackageData, name,self.pkgversion)

    def test_FetchPackageData_nonuser_pkg(self):
        """
        ApMgr-32: Display Description
        @Try to fetch repository package details of nonuser package
        """
        name='hpm-nonuser'
        self._FetchPackageData(name, "") # no exception any more

    def test_FetchPackageDataInstalled_ok(self):
        """
        ApMgr-32: Display Description
        @Try to fetch repository package details of a package
        """
        name='hpm-user1'
        ver='0.1.1-1'

        self.check_package_installed_or_available(name, '', False)

        clear_test_packages_in_apt_archives_dir()
        self._Install(name)
        self.check_install(name, ver)
        self.clear_signals()
        self.check_package_installed_or_available(name, ver, True)

        P = self._FetchPackageDataInstalled(name)
        self.assertEqual(P["Name"], name)
        self.assertEqual(P["Category"], "Desktop")
        self.assertEqual(P["Version"], ver)
        self.assertEqual(P["Description"], "hpm-user1 package\nLong description for package hpm-user1")
        self.assertEqual(P["Maemo-Display-Name-sv_SE"], "Fancy hpm-user1 (sv_SE)")
        self.assertEqual(P["Maemo-Display-Name-de_DE"], "Fancy hpm-user1 (de_DE)")
        self.assertEqual(P["Maemo-Display-Name-fi_FI"], "Fancy hpm-user1 (fi_FI)")
        self.assert_('FlagLocalFile' not in P)

        self._Uninstall(name)
        self.check_uninstall(name)
        self.clear_signals()

    def test_FetchPackageDataInstalled_not_installed(self):
        """
        ApMgr-32: Display Description
        @Try to fetch repository package details of nonexisting package
        """
        self.check_package_not_installed(self.pkgname)
        self.assertRaises(dbus.DBusException, self._FetchPackageDataInstalled, self.pkgname)

    def test_FetchPackageDataInstalled_invalid_pkg(self):
        """
        ApMgr-32: Display Description
        @Try to fetch repository package details of nonexisting package
        """
        name='invalidpackagename'
        self.check_package_not_installed(name)
        self.assertRaises(dbus.DBusException, self._FetchPackageDataInstalled, name)

    def test_FetchPackageDependencyData(self):
        P=['hpm-user-valid-deps']
        V=['0.1.1-1']
        for i in range(5):
            P.append('hpm-user-dep%d' % (i+1))
        for p in P:
            self.check_package_installed_or_available(p, '', False)

        D = self._FetchPackageDependencyData(P[0], '')
        N = set()
        for d in D:
            N.add(d['Name'])
        self.assertEqual(N, set(P))

        self._Install(P[0])
        self.check_install(P[0], V[0], True, None, "", True, True)
        self.clear_signals()

        for p in P:
            self.check_package_installed_or_available(p, V[0], True)

        D = self._FetchPackageDependencyData(P[0], '')
        N = set()
        for d in D:
            N.add(d['Name'])
        self.assertEqual(N, set(P))

        self._Uninstall(P[0])

        # Check that deps are also uninstalled
        for p in P:
            self.check_package_installed_or_available(p, '', False)

    def test_FetchFilesOfInstalledPackage(self):
        p = self.pkgname
        self._Install(p)
        F = self._FetchFilesOfInstalledPackage(p)
        self.assertEqual(len(F), 1)
        self._Uninstall(p)

    def test_fetch_available_updates(self):
        """
        PkMgr-1064
        """
        p1 = 'hpm-user-depends-system-upgrade'
        p2 = 'hpm-user-valid-deps'
        v = '0.1.0-1'
        self._Install(p1, v)
        self.check_install(p1, v)
        self.clear_signals()
        self._Install(p2, v)
        self.check_install(p2, v, True, None, '', True, True)
        self.clear_signals()

        self._RefreshPackageCache("", "")
        self.check_refresh_cache(True)
        self.clear_signals()

        U = self._FetchAvailableUpdates('')
        for pkg in U:
            try:
                pkg['InstalledTimestamp']
            except KeyError:
                pass
            else:
                self.fail('Installed timestamp should not be in the available updates list')
            self.assert_('FlagLocalFile' not in pkg)

    def test_fetch_available_updates_for_failed_installs(self):
        p = 'hpm-user-preinst-fail2'
        v0 = '0.1.0-1'
        v1 = '0.1.1-1'
        
        err = 'OperationFailed'
        open('/etc/testrepo/postinst.status', 'w').write('1')
        self.failUnlessRaisesDBusException(err, self._Install, p, v0)
        self.check_operation_complete("Install", p, v0, err)
        self.check_failed_install(p, v0, None, dbus_error(err))
        self.clear_signals()
        
#        f = open('/var/lib/package-manager/failed-installs', 'r')
#        lines = f.readlines()
#        f.close()
#        for line in lines:
#            print line
        
        self._RefreshPackageCache("", "")
        self.check_refresh_cache(True)
        self.clear_signals()
        
#        pkgs = self._FetchPackages('')
#        for pkg in pkgs:
#            print pkg['Name']

        cnt = 0
        pkgs = self._FetchInstalled()
        for pkg in pkgs:
#            print '%s (%s)' % (pkg['Name'], pkg['Version'])
            if ( pkg['Name'] == p ):
                cnt += 1
                self.assertEqual(pkg['Version'], v0)
            self.assert_('FlagLocalFile' not in pkg)
        self.assertEqual(cnt, 1)
        
        U = self._FetchAvailableUpdates('')
        cnt = 0
        for pkg in U:
#            print '%s (%s)' % (pkg['Name'], pkg['Version'])
            if ( pkg['Name'] == p ):
                cnt += 1
                self.assertEqual(pkg['Version'], v1)
            self.assert_('FlagLocalFile' not in pkg)
        self.assertEqual(cnt, 1)

    def test_FetchAvailableUpdates(self):
        """
        PkMgr-1005
        Return meta-data if reboot is required
        """
        self._RefreshPackageCache("", "")
        self.check_refresh_cache()
        self.clear_signals()

        count = 0
        syspkg_count = 0
        U = self._FetchAvailableUpdates('')
        for pkg in U:
            count += 1
            if ( pkg['Name'] == self.syspkgname ) :
                self.assertEqual(pkg["FlagSystemUpdate"], True)
                syspkg_count += 1
            else:
                self.assert_('FlagReboot' not in pkg)
                self.assert_('FlagSystemUpdate' not in pkg)
            self.assert_('FlagLocalFile' not in pkg)

        self.failIf( count == 0, 'Available updates missing' )
        self.failUnless( syspkg_count == 1, 'No available system upgrades' )

    def test_fetch_available_updates_ignore_one(self):
        """
        PkMgr-??: ??
        @Fetch available updates but ignore one package
        """
        self._RefreshPackageCache("", "clearignored") # Clear the ignored list
        self.check_refresh_cache(True, False)
        self.clear_signals()

        pi_slot = self.iface.connect_to_signal("package_ignored", self.on_package_ignored)

        p = 'hpm-user-upgrade1'
        v = '0.1.1-1'

        self.iface.ignore_update_package(p, v,
                          reply_handler=self.async_reply,
                          error_handler=self.async_error,
                          timeout=self.tmo)
        self.loop.run() # Two loops. Yes, it's in purpose.
        self.loop.run() #
        
        self.assertEqual(self.package_ignored, ('Update', p, v))

        self.clear_signals()
        
        pi_slot.remove()
        
        U = self._FetchAvailableUpdates('')
        count = 0
        for pkg in U:
#            print pkg['Name']
#            print pkg['Version']
#            print ''
            if ( pkg['Name'] == p and pkg['Version'] == v ):
                count += 1
                self.assertEqual( pkg['FlagIgnored'], True )
            self.assert_('FlagLocalFile' not in pkg)

        self.failUnless( count == 1, "The ignored package can not be found" )

        self._RefreshPackageCache("", "clearignored") # Clear the ignored list
        self.check_refresh_cache(True)
        self.clear_signals()


    def on_operation_complete_quit(self, op, pkg, ver, error, reboot):
        self.op_complete_args = (op, pkg, ver, error, reboot)
        self.loop.quit()
    
    def test_fetch_exceptionHandling(self):
        # Some initialization
        op = 'Upgrade'
        p = 'hpm-user-application-upgrade-fail'
        v0 = '0.1.0-1'
        v1 = '0.1.1-1'
        open('/etc/testrepo/postinst.status', 'w').write('0')

        # 1. Install application-upgrade-fail (0.1.0-1)
        self._Install(p, v0)
        self.check_install(p, v0)
        self.clear_signals()
        self.check_package_installed(p)

        # 2. Enable the error
        open('/etc/testrepo/postinst.status', 'w').write('1')

        self._RefreshPackageCache("", "")
        self.check_refresh_cache()
        self.clear_signals()

        # 3. Try to update application-upgrade-fail (0.1.1-1)
        err = 'UpgradeFailed'
        self.failUnlessRaisesDBusException(err, self._Upgrade, p, v1)
        self.check_operation_complete(op, p, v1, err)
        self.clear_signals()

        # 4. Check failures
        self.check_failed_install(p, v1, None, dbus_error(err))
        self.check_failed_upgrade(p, v1, None, dbus_error(err))

        # 5. Cancel update exception for application-upgrade-fail (0.1.1-1)
        oc_slot = self.iface.connect_to_signal("operation_complete", self.on_operation_complete_quit)
        
        self.iface.cancel(op, p, v1)

        err = 'OperationCanceled'
        self.loop.run()
        self.assertEqual(self.op_complete_args, (op, p, v1, dbus_error(err), False))
        self.clear_signals()
        oc_slot.remove()
        
        # 6. Fetch updated packages
        Pkgs = self._FetchAvailableUpdates('')
        cnt = 0
        for pkg in Pkgs:
            if pkg['Name'] == p:
                cnt+=1
                self.assertEqual(pkg['Version'], v1)
                self.assertEqual(pkg['ErrorName'], '')
            self.assert_('FlagLocalFile' not in pkg)
        self.assertEqual(cnt, 1)

        # 7. Fetch data for application-upgrade-fail (0.1.1-1)
        pkg = self._FetchPackageData(p, v1)
        self.assertEqual(pkg['Name'], p)
        self.assertEqual(pkg['Version'], v1)
        self.failIf('ErrorName' in pkg)
        self.assert_('FlagLocalFile' not in pkg)
        
        # 8. Clean up
        open('/etc/testrepo/postinst.status', 'w').write('0')
        self._Uninstall(p)
        self.check_uninstall(p)
        self.clear_signals()
        self._Uninstall('hpm-user-postinst-fail')
        self.check_uninstall('hpm-user-postinst-fail')
        self.clear_signals()


    def test_fetch_exceptionHandling_packageNotFound(self):
        # Some initialization
        p = 'not-existing-package'
        v0 = '0.1.0-1'
        err = 'com.nokia.package_manager.Error.OperationFailed'
        errmsg = 'Operation failed'

#        tc_pkgmgrd_helper.tearDown(self)
        stop_pkgmgrd()
        
        data = """
        <failed-installs>
         <package>
          <error-msg>%s</error-msg>
          <error-code>%s</error-code>
          <version>%s</version>
          <name>%s</name>
         </package>
        </failed-installs>
        """ % (errmsg, err, v0, p)
        
        fn = '/var/lib/package-manager/failed-installs'
        f = open(fn, 'w')
        f.write(data)
        f.close()

        start_pkgmgrd()
        tc_pkgmgrd_helper.setup_dbus_signals(self)
        
#        res = self._FetchPackageData(p, v0)
#        print '- Fetch 1 ------------------'
#        for i in res:
#            print '%s:\t%s' % (i, res[i])
#        print ''
#        self.assertEqual(res['Name'], p)
#        self.assertEqual(res['Version'], v0)

        # Do the fetch
        res = self._FetchPackageDataInstalled(p)
        
        # Check the results
        self.assertEqual(res['Name'], p)
        self.assertEqual(res['Version'], v0)
        self.assertEqual(res['Type'], 'Installed')
        self.assertEqual(res['ErrorMessage'], errmsg)
        self.assertEqual(res['ErrorName'], err)


    def test_fetch_available_updates_nonuser_package_filter(self):
        """
        PkMgr-??: ??
        @Fetch available updates and check that non-user packages are not included
        """
        name = 'hpm-user-upgrade-nonuser'
        ver0 = '0.1.0-1' # User package
#        ver1 = '0.1.1-1' # User package
        ver2 = '0.1.2-1' # Non-user package

        # 1. Install the package
        self._Install(name, ver0)
        self.check_install(name, ver0)
        self.check_package_installed_version(name, ver0)
        self.clear_signals()

        # 2. Check updates list that it does not have the package ver2
        U = self._FetchAvailableUpdates('')
        for pkg in U:
            self.failIf( pkg['Name'] == name and pkg['Version'] == ver2)
            self.assert_('FlagLocalFile' not in pkg)

        # 3. Clean up
        self._Uninstall(name)
        self.check_uninstall(name)
        self.clear_signals()

    def test_FetchPackageDataFile_specialChars(self):
	res = self._FetchPackageDataFile(self.specialcharpkgfilename)
	self.assertEqual(res["Name"], self.specialcharpkgname)
	self.assertEqual(res["Version"], '1.0')
	self.assertEqual(res['FlagLocalFile'], True)


    def test_fetch_listFailedLocalPackage(self):
        # Initializations
        p = 'local-preinst-fail'
        v = '1.0'
        f = get_local_pkgdir() + '/local-preinst-fail-1.0.deb'

        # Preconditions
        self.check_package_not_installed(p)
        
        err = 'OperationFailed'
        open('/etc/testrepo/preinst.status', 'w').write('1')
        self.failUnlessRaisesDBusException(err, self._InstallFile, f)
        self.check_failed_install (p, v, f, dbus_error(err))
        self.clear_signals()
        
        # The fetches
        Pkgs = self._FetchInstalled()
        cnt = 0
        for pkg in Pkgs:
            if ( pkg['Name'] == p ):
                cnt += 1
                self.assertEqual( pkg['Version'], v )
                self.assertEqual( pkg['FlagLocalFile'], True )
                self.assertEqual(pkg['Filename'], f)
        self.assertEqual(cnt, 1)
        
        pkg = self._FetchPackageDataInstalled(p)
        self.assertEqual(pkg['Name'], p)
        self.assertEqual(pkg['Version'], v)
        self.assertEqual(pkg['FlagLocalFile'], True)
        self.assertEqual(pkg['Filename'], f)


# TODO: Disabled. This needs to be enabled when the time comes... ;)
    def disabled_test_fetch_failedUpgrade(self):
        # Some initialization
        p1 = 'hpm-user-postinst-fail'
        p2 = 'hpm-user-application-fail'
        v0 = '0.1.0-1'
        v1 = '0.1.1-1'
        open('/etc/testrepo/postinst.status', 'w').write('0')
        
        # 1. Install hpm-user-postinst-fail (0.1.0-1)
        self._Install(p1, v0)
        self.check_install(p1, v0)
        self.clear_signals()
        self.check_package_installed(p1)
        
        # 2. Install hpm-user-application-fail (0.1.0-1)
        self._Install(p2, v0)
        self.check_install(p2, v0)
        self.clear_signals()
        self.check_package_installed(p2)
        
        # 3. Enable the error
        open('/etc/testrepo/postinst.status', 'w').write('1')
        
        self._RefreshPackageCache("", "")
        self.check_refresh_cache()
        self.clear_signals()

        # 4. Try to update hpm-user-postinst-fail to 0.1.1-1
        err = 'UpgradeFailed'
        self.failUnlessRaisesDBusException(err, self._Upgrade, p1, v1)
        self.check_operation_complete("Upgrade", p1, v1, err)
        self.clear_signals()
        
        # 5. Check failures
        self.check_failed_install(p1, v1, None, dbus_error(err))
        self.check_failed_upgrade(p1, v1, None, dbus_error(err))
        self.check_failed_install(p2, '', None, '', False)
        self.check_failed_upgrade(p2, '', None, '', False)

        # 6. Check what fetchAvailableUpdates() contains
        Pkgs = self._FetchAvailableUpdates('')
        cnt = 0
        for pkg in Pkgs:
            self.assertEqual(pkg['Type'], 'Update')
            self.assert_('FlagLocalFile' not in pkg)
            self.failIf( pkg['Name'] == p2 )
            if ( pkg['Name'] == p1 ):
                cnt += 1
                self.assertEqual(pkg['ErrorName'], dbus_error(err))
                self.assertEqual(pkg['Version'], v1)
#            print ''
#            print '- %s --------------------------------------------------------------------------' % pkg['Name']
#            for i in pkg:
#                print "%s:\t%s" % (i, pkg[i])
        self.assertEqual(cnt, 1)

        # 7. Clean up
        open('/etc/testrepo/postinst.status', 'w').write('0')

        self._Uninstall(p2)
        self.check_uninstall(p2)
        self.clear_signals()
        
        self._Uninstall(p1)
        self.check_uninstall(p1)
        self.clear_signals()

