# -*- python -*-
#
# Copyright (C) 2009 - 2011 Nokia Corporation.
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License version
# 2 as published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see http://www.gnu.org/licenses/.

#
# TODO: Check exception strings?

import unittest
import dbus
import commands
import sys

from test_helper import *

# whether to check fetching lists fully or only our test repo packages
full_fetch_check = False

def dbus_error(str):
    return 'com.nokia.package_manager.Error.'+str

class test_daemon_dbus_install_file(tc_pkgmgrd_helper):
    def setUp(self):
        tc_pkgmgrd_helper.setUp(self)
        pkgdir = get_pkgdir()
        arch = deb_arch()
        userdir = get_repodir()+'/pool/user/development'
        maindir = get_repodir()+'/pool/main/misc'
        self.pkgname = 'hpm-user1'
        self.pkgversion = '0.1.0-1'
        self.pkgfilename = pkg_filename(get_repodir()+'/pool/user/desktop', self.pkgname)
        self.nonuserpkgname = 'hpm-nonuser'
        self.nonuserpkgfilename = pkg_filename(maindir, self.nonuserpkgname)
        self.invalidarchpkgname = 'hpm-user-invalid-arch'
        self.invalidarchpkgfilename = pkg_filename(userdir, self.invalidarchpkgname, self.pkgversion, 'powerpc')
        self.missingdepspkgname = 'hpm-user-missing-deps'
        self.missingdepspkgfilename = pkg_filename(userdir, self.missingdepspkgname)
        self.dependencypkgname = 'hpm-user-dep'
        self.dependencypkgfilename = pkg_filename(userdir, self.dependencypkgname)
        self.nonuserdependencypkgname = 'hpm-nonuser-dep'
        self.nonuserdependencypkgfilename = pkg_filename(userdir, self.nonuserdependencypkgname)
        self.validdepspkgname = 'hpm-user-valid-deps'
        self.validdepspkgversion = '0.1.0-1'
        self.validdepspkgfilename = pkg_filename(userdir, self.validdepspkgname)
        self.localpkgname = 'local-package'
        self.localpkgversion = '1.0'
        self.localpkgfilename = get_local_pkgdir() + '/local-package-1.0.deb'
        self.local_corrupted_pkg = 'local-corrupted-package'
        self.local_corrupted_pkgver = '1.0'
        self.local_corrupted_pkgfile = get_local_pkgdir() + '/local-corrupted-package-1.0.deb'
        self.localpercentpkgname = 'local-package-percent'
        self.localpercentpkgversion = '1.0'
        self.localpercentpkgfilename = get_local_pkgdir() + '/local-package-%-1.0.deb'
        
        self.local_pkg_requires_os_upgrade = 'local-package-requires-os-upgrade'
        self.local_pkgver_requires_os_upgrade = '1.0'
        self.local_pkgfile_requires_os_upgrade = get_local_pkgdir() + '/local-package-requires-os-upgrade-1.0.deb'

    def tearDown(self):
        tc_pkgmgrd_helper.tearDown(self)

    def test_2_0_InstallFile_not_repo(self):
        """
        ApMgr-X: Install a standalone package that is not in the repository. 
        @Install package from file system
        """
        p = self.localpkgname
        v = self.localpkgversion
        f = self.localpkgfilename

        self.failUnless(os.path.isfile(f))
        self.check_package_not_installed(p)
        self.check_package_not_available(p)

        self._InstallFile(f)
        self.check_install(p, v, False, f)
        self.clear_signals()
        self.check_package_installed(p)

        # Check that reinstalling the same file works.
        self._InstallFile(f)
        self.check_install(p, v, False, f)
        self.clear_signals()
        self.check_package_installed(p)

        self._Uninstall(p)
        self.check_uninstall(p)
        self.clear_signals()

    def test_2_1_InstallFile_ok(self):
        """
        ApMgr-3: Install a Standalone Package
        @Install package from file system
        """
        p = self.pkgname
        v = self.pkgversion
        f = self.pkgfilename

        self.failUnless(os.path.isfile(f))
        self.check_package_installed_or_available(p, '', False, False, f)

        self._InstallFile(f)
        self.check_install(p, v, False, f)
        self.clear_signals()
        self.check_package_installed_or_available(p, v, True, False, f)

        # Check that reinstalling the same file works
        self._InstallFile(f)
        self.check_install(p, v, False, f)
        self.clear_signals()
        self.check_package_installed_or_available(p, v, True, False, f)

        self._Uninstall(p)
        self.check_uninstall(p)
        self.clear_signals()
        self.check_package_installed_or_available(p, '', False, False, f)

    def test_2_2_InstallFile_deps(self):
        """
        ApMgr-3: Install a Standalone Package
        @ Try to Install package from file system with deps that are
        not installed
        """
        p = self.validdepspkgname
        v = self.validdepspkgversion
        f = self.validdepspkgfilename
        D = []
        for i in range(5):
            D.append('hpm-user-dep%d' % (i+1))

        self.failUnless(os.path.isfile(f))
        self.check_package_not_installed(p)
        for d in D:
            self.check_package_not_installed(d)

        self._InstallFile(f)

        self.check_install(p, v, False, f, many_complete = True, allow_download = True)
        self.clear_signals()

        self.check_package_installed(p)
        for d in D:
            self.check_package_installed(d)

    def test_2_3_InstallFile_fail_file_not_found(self):
        """
        ApMgr-3: Install a Standalone Package
        * Try to install package with invalid filename
        """
        p = 'hpm-nonexisting-package'
        f = '/hpm-nonexisting-package_0.1.0-1_i386.deb'
        err = 'FileNotFound'

        self.failIf(os.path.isfile(f))

        self.failUnlessRaisesDBusException(err, self._InstallFile, f)

        self.check_operation_complete('InstallFile', f, None, err)
        self.clear_signals()
        self.check_package_not_installed(p)

    def test_2_4_InstallFile_fail_wrong_category(self):
        """
        ApMgr-3: Install a Standalone Package
        @ Try to Install nonuser package from file system
        """
        p = self.nonuserpkgname
        f = self.nonuserpkgfilename
        err = 'PackageNotUser'

        self.failUnless(os.path.isfile(f))
        self.check_package_not_installed(p)

        self.failUnlessRaisesDBusException(err, self._InstallFile, f)

        self.check_package_not_installed(p)
        self.check_operation_complete("InstallFile", f, None, err)
        self.clear_signals()

    def test_2_5_InstallFile_fail_wrong_architecture(self):
        """
        ApMgr-3: Install a Standalone Package
        @ Try to Install package from file system with incompatible arch
        """
        p = self.invalidarchpkgname
        f = self.invalidarchpkgfilename
        err = 'PackageNotCompatible'

        self.failUnless(os.path.isfile(f))
        self.check_package_not_installed(p)

        self.failUnlessRaisesDBusException(err, self._InstallFile, f)

        self.check_package_not_installed(p)
        self.check_operation_complete("InstallFile", f, None, err)
        self.clear_signals()

    def test_2_6_InstallFile_fail_invalid_file(self):
        """
        ApMgr-3: Install a Standalone Package
        @ Try to Install non debian package from file system
        """
        f = '/etc/group'
        err = 'InvalidFile'

        self.failUnless(os.path.isfile(f))

        self.failUnlessRaisesDBusException(err, self._InstallFile, f)

        self.check_operation_complete("InstallFile", f, None, err)
        self.clear_signals()

    def test_2_7_InstallFile_fail_missing_dependencies(self):
        """
        ApMgr-3: Install a Standalone Package
        @ Try to Install package from file system with missing deps
        """
        p = self.missingdepspkgname
        f = self.missingdepspkgfilename
        err = 'MissingDependencies'

        self.failUnless(os.path.isfile(f))
        self.check_package_installed_or_available(p, '', False)

        self.failUnlessRaisesDBusException(err, self._InstallFile, f)

        self.check_operation_complete("InstallFile", f, None, err)
        self.clear_signals()
        self.check_package_not_installed(p)

    def test_install_file_operation_fail(self):
        p = 'hpm-user-preinst-fail'
        v = '0.1.0-1'
        f = '%s/%s/%s' % (get_repodir(), '/pool/user/development', 'hpm-user-preinst-fail_0.1.0-1_all.deb')
        self.failUnless(os.path.isfile(f))

        err = 'OperationFailed'
        self.check_package_installed_or_available(p, '', False)
        self.failUnlessRaisesDBusException(err, self._InstallFile, f)

        self.check_operation_complete("InstallFile", f, None, err)
        self.clear_signals()
#        self.check_package_installed_or_available(p, '', False)
        self.check_failed_install(p, '0.1.0-1', f, dbus_error(err))

        P = self._FetchPackageDataFile(f)
        self.assertEqual(P['ErrorName'], dbus_error(err))
        
        pkgs = self._FetchInstalled()
        cnt = 0
        for pkg in pkgs:
            if pkg['Name'] == p :
                self.assertEqual(pkg['ErrorName'], dbus_error(err))
                cnt += 1
        self.failUnless(cnt == 1) # Ensure that the package was on the list

    def test_install_file_not_repo_operation_fail(self):
        p = 'local-preinst-fail'
        v = '1.0'
        f = get_local_pkgdir() + '/local-preinst-fail-1.0.deb'
        self.failUnless(os.path.isfile(f))

        err = 'OperationFailed'
        self.check_package_not_installed(p)
        self.check_package_not_available(p)
        self.failUnlessRaisesDBusException(err, self._InstallFile, f)

        self.check_operation_complete("InstallFile", f, None, err)
        self.clear_signals()
        self.check_package_not_installed(p)
        self.check_package_not_available(p)
        self.check_failed_install(p, v, f, dbus_error(err))

        P = self._FetchPackageDataFile(f)
        self.assertEqual(P['ErrorName'], dbus_error(err))
        
        P = self._FetchPackageDataInstalled(p)
        self.assertEqual(P['ErrorName'], dbus_error(err))
        
        pkgs = self._FetchInstalled()
        cnt = 0
        for pkg in pkgs:
            if pkg['Name'] == p :
                self.assertEqual(pkg['ErrorName'], dbus_error(err))
                cnt += 1
        self.failUnless(cnt == 1) # Ensure that the package was on the list
        
        pkgs = self._FetchAvailableUpdates('')
        for pkg in pkgs:
            self.failIf( pkg['Name'] == p ) # Check that the local package is not in the updates list

    def test_install_file_requires_system_upgrade(self):
        p = 'hpm-user-depends-system-upgrade'
        v0 = '0.1.0-1'
        v1 = '0.1.1-1'

        system('update-testrepo')
        self._RefreshPackageCache("", "clearignored")

        f = '%s/%s/%s' % (get_repodir(), '/pool/user/development', 'hpm-user-depends-system-upgrade_%s_%s.deb' % (v1, deb_arch()))
        self.assert_(os.path.isfile(f))

        err = 'RequiresSystemUpgrade'
        self.failUnlessRaisesDBusException(err, self._InstallFile, f)

        self._Install(p, v0)

        err = 'RequiresSystemUpgrade'
        self.failUnlessRaisesDBusException(err, self._InstallFile, f)


    def test_install_file_corrupted(self):
        # Initializations
        p = self.local_corrupted_pkg
        v = self.local_corrupted_pkgver
        f = self.local_corrupted_pkgfile

        self.failUnless(os.path.isfile(f))
        self.check_package_not_installed(p)
        self.check_package_not_available(p)

        err = 'InvalidFile'
        self.failUnlessRaisesDBusException(err, self._InstallFile, f)
        self.failUnlessRaisesDBusException(err, self._FetchPackageDataFile, f)

        pkgs = self._FetchPackages('')
        cnt = 0
        for pkg in pkgs:
            if ( pkg['Name'] == p ):
                cnt += 1
        self.failUnless( cnt == 0 )

    def test_install_file_requires_os_upgrade(self):
        # Initializations
        p = self.local_pkg_requires_os_upgrade
        v = self.local_pkgver_requires_os_upgrade
        f = self.local_pkgfile_requires_os_upgrade

        system('update-testrepo')
        self._RefreshPackageCache("", "clearignored")
        self.clear_signals()

        self.failUnless(os.path.isfile(f))
        self.check_package_not_installed(p)
        self.check_package_not_available(p)

        err = 'RequiresSystemUpgrade'
        self.failUnlessRaisesDBusException(err, self._InstallFile, f)
        
        self.check_failed_install(p, v, f, dbus_error(err))
        P = self._FetchPackageDataFile(f)
        self.assertEqual(P['ErrorName'], dbus_error(err))
        self.assertEqual(P['Filename'], f)

        self.assertEqual(P['DependsSystemUpgradeName'], 'hpm-system-meta')
        self.assertEqual(P['DependsSystemUpgradeVersion'], '0.1.2-1')

        pkgs = self._FetchPackages('')
        cnt = 0
        for pkg in pkgs:
            if ( pkg['Name'] == p ):
                cnt += 1
        self.failUnless( cnt == 0 )

    def test_install_file_error_tracking(self):
        # Errors are only tracked for first installations and for
        # failed updates that leave the package broken.

        userdir = get_repodir()+'/pool/user/development'
        
        # Failed first installation
        #
        p = 'hpm-user-postinst-fail'
        v = '0.1.0-1'
        f = pkg_filename (userdir, p, v, 'all')
        self.check_package_not_installed(p);
        open('/etc/testrepo/postinst.status', 'w').write('1')
        self.failUnlessRaisesDBusException('OperationFailed', self._InstallFile, f)
        self.check_failed_install (p, v, f, dbus_error ('OperationFailed'))
        self.clear_signals()

        # Successful
        #
        open('/etc/testrepo/postinst.status', 'w').write('0')
        self._InstallFile(f)
        self.check_failed_install (p, v, f, '', False)
        self.clear_signals()
        self.check_package_installed(p)
                
        # Failed upgrade/reinstall, should leave package in broken
        # state.
        #
        open('/etc/testrepo/postinst.status', 'w').write('1')
        self.failUnlessRaisesDBusException('OperationFailed', self._InstallFile, f)
        self.check_failed_install (p, v, f, dbus_error ('OperationFailed'))
        self.clear_signals()
        
        # Cleanup
        #
        self._Uninstall(p)
        self.check_uninstall(p)
        self.clear_signals()

        # Success first installation of another package
        #
        p = 'hpm-user-preinst-fail'
        v = '0.1.0-1'
        f = pkg_filename (userdir, p, v, 'all')
        self.check_package_not_installed(p);
        open('/etc/testrepo/preinst.status', 'w').write('0')
        self._InstallFile(f)
        self.check_failed_install (p, v, f, '', False)
        self.clear_signals()
        self.check_package_installed(p)

        # Failed upgrade/reinstall, doesn't leave package in broken
        # state
        #
        open('/etc/testrepo/preinst.status', 'w').write('1')
        self.failUnlessRaisesDBusException('OperationFailed', self._InstallFile, f)
        self.check_failed_install (p, v, f, '', False)
        self.clear_signals()        

        # Cleanup
        #
        self._Uninstall(p)
        self.check_uninstall(p)
        self.clear_signals()


    def test_InstallFile_percent_mark_in_its_filename(self):
        p = self.localpercentpkgname
        v = self.localpercentpkgversion
        f = self.localpercentpkgfilename

        self.failUnless(os.path.isfile(f))
        self.check_package_not_installed(p)
        self.check_package_not_available(p)

        self._InstallFile(f)
        self.check_install(p, v, False, f)
        self.clear_signals()
        self.check_package_installed(p)

        # Check that reinstalling the same file works.
        self._InstallFile(f)
        self.check_install(p, v, False, f)
        self.clear_signals()
        self.check_package_installed(p)

        self._Uninstall(p)
        self.check_uninstall(p)
        self.clear_signals()


