# -*- python -*-
#
# Copyright (C) 2009 - 2011 Nokia Corporation.
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License version
# 2 as published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see http://www.gnu.org/licenses/.


import unittest
import dbus
import commands
import sys
import datetime

from test_helper import *

def pkg_filename(path, name, version='0.1.0-1', arch=deb_arch()):
    return '%s/%s_%s_%s.deb' % (path, name, version, arch)

def dbus_error(str):
    return 'com.nokia.package_manager.Error.'+str

class test_daemon_dbus_refresh(tc_pkgmgrd_helper):
    def setUp(self):
#        print "Setup"
        self.available_packages = '/var/lib/package-manager/available-packages'
#        self.ignoredlist = '/var/lib/package-manager/ignored-packages'
        tc_pkgmgrd_helper.setUp(self)
        setup_testrepo_upgrade_packages()
        pkgdir = get_pkgdir()
        arch = deb_arch()

        if ( os.path.exists(self.available_packages) ):
            os.remove(self.available_packages)

        self._RefreshPackageCache("", "clearignored")
        self.check_operation_complete("Refresh", "")
        self.clear_signals()
        
        for i in range(3):
            n = i+1
            system("pkgmgr uninstall -p hpm-user%d" % n, False)
            system("pkgmgr install -p hpm-user%d -v %s" % (n, '0.1.0-1'))

    def tearDown(self):
        for i in range(3):
            n = i+1
            system("pkgmgr uninstall -p hpm-user%d" % n, False)
        tc_pkgmgrd_helper.tearDown(self)
#        print "Teardown"
        

    def test_refresh_cache_ok(self):
        v0 = '0.1.0-1'
        v1 = '0.1.1-1'
        P = []
        for i in range(3):
            P.append('hpm-user%d' % (i+1))

        self.downgrade_testrepo()

        self._RefreshPackageCache("", "clearignored")
        self.check_operation_complete("Refresh", "")
        self.clear_signals()

        setup_testrepo_upgrade_packages()

        ts_before = now()
        time.sleep(1)
        self._RefreshPackageCache("", "clearignored")
        self.check_refresh_cache()
        self.clear_signals()
        time.sleep(1)
        ts_after = now()

        U = self._FetchAvailableUpdates('')
#        self.assertEqual(len(U), 3)
        count = 0
        for up in U:
            if ( up['Name'] in P ):
                count += 1
#            self.failUnless(up['Name'] in P)
                self.assertEqual(up['Version'], v1)
            try:
                up['InstalledTimestamp']
            except KeyError:
                pass
            else:
                self.fail('Installed timestamp should not be in the available updates list')
            try:
                up['AvailableTimestamp']
            except KeyError:
                self.fail('Available timestamp is missing from the available updates list')
                pass
            else:
                ts = datetime.datetime.fromtimestamp( up['AvailableTimestamp'] )
                self.failIf( ts_before > ts ,'Available timestamp is older than it should')
                self.failIf( ts_after < ts ,'Available timestamp is newer than it should')
            
        self.failUnless( count == 3 )
        
        self.failUnless(os.path.exists(self.available_packages))
        f = open( self.available_packages, "r")
        text = f.read()
        f.close()
        self.failIf( text == '' )


    def test_refresh_cache_remove_updates_from_repo(self):
        v0 = '0.1.0-1'
        v1 = '0.1.1-1'
        P = []
        for i in range(3):
            P.append('hpm-user%d' % (i+1))

        self._RefreshPackageCache("", "clearignored")
        self.check_operation_complete("Refresh", "")
        self.clear_signals()

        U = self._FetchAvailableUpdates('')
        count = 0
        for up in U:
            if ( up['Name'] in P ):
                count += 1
                self.assertEqual(up['Version'], v1)

        self.failUnless( count == 3 )

        self.failUnless(os.path.exists(self.available_packages))
        f = open( self.available_packages, "r")
        text = f.read()
        f.close()

        self.failIf( text == '' )
 
        count = 0       
        tokens = text.split('\n')
        for s in tokens:
#            print 's: %s' % s
            p = s.split()
            if ( p == [] ):
                continue
            if ( p[0] in P ) and ( v1 in p[1] ):
                count += 1
        self.failUnless( count == 3 )

        self.downgrade_testrepo()

        self._RefreshPackageCache("", "clearignored")
        self.check_operation_complete("Refresh", "")
        self.clear_signals()

        count = 0

        U = self._FetchAvailableUpdates('')
        count = 0
        for up in U:
            count += 1
            if ( up['Name'] in P ):
                self.failIf(up['Version'] == v1)

        self.failUnless( count == 0 )
        
        self.failUnless(os.path.exists(self.available_packages))
        f = open( self.available_packages, "r")
        text = f.read()
        f.close()

        self.failUnless( text == '' )
 
#        count = 0
#        tokens = text.split('\n')
#        for s in tokens:
#            print 's: %s' % s
#            p = s.split()
#            print p
#            if ( p[0] in P ):
#                count += 1
#                self.failIf( v1 in s )

#        self.failUnless( count == 3 )
#        self.failUnless( tokens[0] == p )
#        self.failUnless( tokens[1] == ('v=%s') % v )

    def test_refresh_cache_notifies_updates_for_failed_installs(self):
        self._RefreshPackageCache("", "clearignored")
        self.clear_signals()

        err = 'OperationFailed'
        self.failUnlessRaisesDBusException(err, self._Install, 'hpm-user-fail', '0.1.0-1')
        self.clear_signals()

        self._RefreshPackageCache("", "clearignored")
        # Check that updates are available
        self.check_refresh_cache(True)

        # Clean up
        system('stop-pkgmgrd')
        remove(self.available_packages)
        system('start-pkgmgrd')

    def test_refresh_cache_partial(self):
        # Refresh only the 'other' part of the testrepo
        self.downgrade_testrepo()

        self._RefreshPackageCache("", "clearignored")
        self.check_operation_complete("Refresh", "")
        self.clear_signals()

        system("pkgmgr install -p hpm-user-cert -v 0.1.0-1")

        U = self._FetchAvailableUpdates('')
        self.failUnless (len(U) == 0,
                         "No updates should be available, but there were %d" % len(U))

        setup_testrepo_upgrade_packages()

        self._RefreshPackageCache("com.nokia.maemo/testrepo-other", "clearignored")
        self.check_operation_complete("Refresh", "")
        self.clear_signals()

        U = self._FetchAvailableUpdates('')
        self.failUnless (len(U) == 1 and U[0]['Name'] == 'hpm-user-cert',
                         "Update for hpm-user-cert should be available, and no others.\n" +
                         "(There were %d available updates)" % len(U))

    def test_refresh_cache_partial_success(self):
        # Only some of the sources will succeed.
        
        fn = '/etc/apt/sources.list.d/broken.list'
        f = open(fn, 'w')
        f.write ('deb http://127.0.0.1:8080/ non-existing user\n');
        f.close()

        try:
            self.failUnlessRaisesDBusException('PartialSuccess', self._RefreshPackageCache, '', '')
            self.clear_signals()
        finally:
            os.unlink (fn)

    def test_refresh_cache_full_failure(self):
        # Every single source will fail, individually.
        
        d = get_repodir()
        system ("rm -rf " + d + ".disabled")
        os.rename (d, d + '.disabled')

        try:
            self.failUnlessRaisesDBusException('OperationFailed', self._RefreshPackageCache, '', '')
            self.clear_signals()
        finally:
            os.rename (d + '.disabled', d);

    def async_reply(self):
        self.loop.quit()

    def async_error(self, e):
        self.error = e
        self.loop.quit()

    def on_package_ignored(self, ign_type, pkg, ver):
        self.package_ignored = (ign_type, pkg, ver)
        self.loop.quit()

    def test_refresh_clear_ignored(self):
        self._RefreshPackageCache("", "clearignored") # Clear the ignored list
        self.check_operation_complete("Refresh", "")
        self.clear_signals()

        pi_slot = self.iface.connect_to_signal("package_ignored", self.on_package_ignored)

        p = 'hpm-user-upgrade1'
        v = '0.1.1-1'

        self.iface.ignore_update_package(p, v,
                          reply_handler=self.async_reply,
                          error_handler=self.async_error,
                          timeout=self.tmo)
        self.loop.run() # Two loops. Yes, it's in purpose.
        self.loop.run() #
        
        self.assertEqual(self.package_ignored, ('Update', p, v))

        self.clear_signals()
        
        pi_slot.remove()
        
        U = self._FetchAvailableUpdates('')
        count = 0
        for pkg in U:
            if ( pkg['Name'] == p and pkg['Version'] == v ):
                count += 1
                self.assertEqual( pkg['FlagIgnored'], True )

        self.failUnless( count == 1, "The ignored package can not be found" )

        self._RefreshPackageCache("", "clearignored") # Clear the ignored list
        self.check_refresh_cache(True, True)
        self.clear_signals()



