# -*- python -*-
#
# Copyright (C) 2009 - 2011 Nokia Corporation.
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License version
# 2 as published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see http://www.gnu.org/licenses/.

#
# TODO: Check exception strings?

import unittest
import dbus
import commands
import sys

from test_helper import *

# whether to check fetching lists fully or only our test repo packages
full_fetch_check = False

#def pkg_filename(path, name, version='0.1.0-1', arch=deb_arch()):
#    return '%s/%s_%s_%s.deb' % (path, name, version, arch)

def dbus_error(str):
    return 'com.nokia.package_manager.Error.'+str

class test_daemon_dbus_uninstall(tc_pkgmgrd_helper):
    def setUp(self):
        tc_pkgmgrd_helper.setUp(self)
        pkgdir = get_pkgdir()
        arch = deb_arch()
        userdir = get_repodir()+'/pool/user/development'
        maindir = get_repodir()+'/pool/main/misc'
        self.pkgname = 'hpm-user2'
        self.pkgversion = '0.1.0-1'
        self.pkgfilename = pkg_filename(userdir, self.pkgname)
        self.nonuserpkgname = 'hpm-nonuser'
        self.nonuserpkgfilename = pkg_filename(maindir, self.nonuserpkgname)
        self.invalidarchpkgname = 'hpm-user-invalid-arch'
        self.invalidarchpkgfilename = pkg_filename(userdir, self.invalidarchpkgname, self.pkgversion, 'powerpc')
        self.missingdepspkgname = 'hpm-user-missing-deps'
        self.missingdepspkgfilename = pkg_filename(userdir, self.missingdepspkgname)
        self.dependencypkgname = 'hpm-user-dep'
        self.dependencypkgfilename = pkg_filename(userdir, self.dependencypkgname)
        self.nonuserdependencypkgname = 'hpm-nonuser-dep'
        self.nonuserdependencypkgfilename = pkg_filename(userdir, self.nonuserdependencypkgname)
        self.validdepspkgname = 'hpm-user-valid-deps'
        self.validdepspkgfilename = pkg_filename(userdir, self.validdepspkgname)
        self.localpkgname = 'localtestpkg'
        self.localpkgfilename = pkg_filename(userdir, self.localpkgname)

    def tearDown(self):
        tc_pkgmgrd_helper.tearDown(self)

    def test_Uninstall_nonexisting_pkg(self):
        """
        ApMgr-7: Remove Package
        * Try to uninstall non-existing repository package
        """
        name='invalidpackagename'
        self.check_package_not_installed(name)

        self.assertRaises(dbus.DBusException, self._Uninstall, name)
        self.check_operation_complete("Uninstall", name, None, "PackageNotFound")
        self.clear_signals()

    def test_Uninstall_notinstalled_pkg(self):
        """
        ApMgr-7: Remove Package
        * Try to uninstall repository package that is not installed
        """
        self.check_package_not_installed(self.pkgname)
        self.assertRaises(dbus.DBusException, self._Uninstall, self.pkgname)
        self.check_operation_complete("Uninstall", self.pkgname, None, "PackageNotInstalled")
        self.clear_signals()

    def test_Uninstall_nonuser_pkg(self):
        """
        ApMgr-7: Remove Package
        * Try to remove no-user package
        """
        name='hpm-nonuser-upgrade'
        self.check_package_installed(name)

        self.assertRaises(dbus.DBusException, self._Uninstall, name)
        self.check_package_installed(name)
        self.check_operation_complete("Uninstall", name, None, "PackageNotUser")
        self.clear_signals()
        
    def test_Uninstall_dependency_pkg(self):
        """
        ApMgr-7: Remove Package
        * Try to remove a dependency package
        """
        pkg = 'hpm-user-valid-deps'
        version = '0.1.0-1'
        self.check_package_not_installed(pkg)
        
        deppkg = 'hpm-user-dep3'
        depversion = version
        
# check_install(self, package, version, check_download=True, filename=None, error="", started=True, many_complete=False, allow_download=False):
        self._Install(pkg)
        self.check_install(pkg, version, many_complete=True)
        self.clear_signals()
        self.check_package_installed(pkg)
        
        self.check_package_installed(deppkg)

        err = 'DependenciesPreventUninstall'
        self.failUnlessRaisesDBusException(err, self._Uninstall, deppkg)
#        self._Uninstall(deppkg)
        self.clear_signals()
        self.check_package_installed_or_available(deppkg, depversion, True)

        self._Uninstall(pkg)
        self.check_uninstall(pkg, True)
        self.clear_signals()
        self.check_package_installed_or_available(pkg, version, False)
        self.check_package_installed_or_available(deppkg, depversion, False)
