# -*- python -*-
#
# Copyright (C) 2009 - 2011 Nokia Corporation.
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License version
# 2 as published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see http://www.gnu.org/licenses/.

#
# TODO: Check exception strings?

import unittest
import dbus
import commands
import sys

from test_helper import *

class test_daemon_dbus_upgrade(tc_pkgmgrd_helper):
    def setUp(self):
        tc_pkgmgrd_helper.setUp(self)
        setup_testrepo_upgrade_packages()
        pkgdir = get_testdir()
        arch = deb_arch()
        self.v0 = '0.1.0-1'
        self.v1 = '0.1.1-1'
        self.pkgversion = '0.1.0-1'
        self.update_pkg_version = "0.1.1-1"

        self.hpm_user_noupgrade = "hpm-user-upgrade-no-upgrade"
        self.hpm_user_upgrade1 = 'hpm-user-upgrade1'
#         self.hpm_user_upgrade1_file = ("%s/%s") % (pkgdir, 'hpm-user-upgrade1_0.1.0-1_'+arch+'.deb')
        self.hpm_nonuser_upgrade_dep = 'hpm-nonuser-upgrade-dep'
#         self.hpm_nonuser_upgrade_dep_file = ("%s/%s") % (pkgdir, 'hpm-nonuser-upgrade-dep_0.1.0-1_'+arch+'.deb')
        self.hpm_user_invalid_arch = 'hpm-user-invalid-arch'
#         self.hpm_user_invalid_arch_file = ("%s/%s") % (pkgdir, 'hpm-user-invalid-arch_0.1.0-1_powerpc.deb')
        self.hpm_user_upgrade_missing_deps = 'hpm-user-upgrade-missing-deps'
#         self.hpm_user_upgrade_missing_deps_file = ("%s/%s") % (pkgdir, 'hpm-user-upgrade-missing-deps_0.1.1-1_'+arch+'.deb')
        disable_reboot_script()

    def tearDown(self):
        tc_pkgmgrd_helper.tearDown(self)

    def test_4_1_Upgrade_ok(self):
        """
        ApMgr-??: ??
        @Upgrade package
        """
        p = self.hpm_user_upgrade1
        v = self.v0

        self.check_package_installed_or_available(p, v, True, True)

        self._RefreshPackageCache("", "")
        self.check_refresh_cache()
        self.clear_signals()

        # On second time there should not be any updates -> no packageListUpdated signal.
        self._RefreshPackageCache("", "")
        self.check_refresh_cache(False)
        self.clear_signals()

        self._Upgrade(p)
        self.check_upgrade(p, self.v1, False)
        self.clear_signals()

        is_package_installed_version(p, self.v1)

        U = self._FetchAvailableUpdates('')
        N = []
        for u in U:
            N.append(u['Name'])
        self.failIf(p in N)

    def test_4_2_Upgrade_ok_valid_dependencies(self):
        p = 'hpm-user-upgrade-valid-deps'
        A = [p]
        for i in range(5):
            A.append('hpm-user-upgrade-dep%d' % (i+1))

        self.check_packages_installed_or_available(A, True)
        for a in A:
            is_package_installed_version(a, self.v0)

        self._RefreshPackageCache("", "")
        self.check_refresh_cache(False)
        self.clear_signals()
        self._Upgrade(p)
        self.check_upgrade(p, self.v1, False)
        self.clear_signals()

        self.check_packages_installed_or_available(A, True)
        for a in A:
            is_package_installed_version(a, self.v1)

        U = self._FetchAvailableUpdates('')
        N = []
        for u in U:
            N.append(u['Name'])
#        for a in A:
#            self.failIf(a in N)

    def test_4_4_Upgrade_fail_missing_dependencies(self):
        """
        ApMgr-5: Upgrade Package from configured repositories
        @ Try to Upgrade repository package with missing deps
        """
        p = 'hpm-user-upgrade-missing-deps'
        err = 'MissingDependencies'

        self.check_package_installed_or_available(p, self.v0, True, True)
        is_package_installed_version(p, self.v0)

        self._RefreshPackageCache("", "")
        self.check_refresh_cache(False)
        self.clear_signals()
        self.failUnlessRaisesDBusException(err, self._Upgrade, p)

        self.check_operation_complete("Upgrade", p, None, err)
        self.clear_signals()
        self.check_package_installed_or_available(p, self.v0, True, True)
        is_package_installed_version(p, self.v0)

    def test_4_5_Upgrade_fail_package_not_found(self):
        """
        ApMgr-??: ??
        * Try to upgrade non-existing package
        """
        p = 'hpm-nonexisting-package'
        err = 'NoCandidate'

        self.check_package_not_installed(p)

        self.failUnlessRaisesDBusException(err, self._Upgrade, p)

        self.check_package_not_installed(p)
        self.check_operation_complete("Upgrade", p, None, err)
        self.clear_signals()

        self.check_package_not_installed(p)

    def test_4_6_Upgrade_fail_wrong_category(self):
        """
        ApMgr-??: ??
        * Try to upgrade nonuser repository package
        """
        p = 'hpm-nonuser-upgrade'
        err = 'PackageNotUser'

        is_package_installed_version(p, self.v0)

        self.failUnlessRaisesDBusException(err, self._Upgrade, p)

        self.check_operation_complete("Upgrade", p, None, err)
        self.clear_signals()
        is_package_installed_version(p, self.v0)

    def test_4_7_Upgrade_fail_wrong_architecture(self):
        """
        ApMgr-??: ??
        * Try to upgrade repository package with incompatible arch
        """
        p = 'hpm-user-invalid-arch'
        err = 'NoCandidate'

        self.check_package_not_installed(p)

        self.failUnlessRaisesDBusException(err, self._Upgrade, p)

        self.check_operation_complete("Upgrade", p, None, err)
        self.clear_signals()
        self.check_package_not_installed(p)

    def test_Upgrade_specific_version(self):
        p = 'hpm-user-upgrade2'
        v0 = '0.1.0-1'
        v1 = '0.1.1-1'

        self._Install(p, v0)
        self.check_install(p, v0)
        self.clear_signals()

        self._RefreshPackageCache("", "")
        self.check_refresh_cache(True)
        self.clear_signals()
        
        self._Upgrade(p, v1)
        self.check_upgrade(p, v1)
        self.clear_signals()

        err = 'PackageAlreadyNewest'
        self.failUnlessRaisesDBusException(err, self._Upgrade, p, v1)
        self.clear_signals()

    def test_Upgrade_fail_requires_system_upgrade(self):
        p = 'hpm-user-depends-system-upgrade'
        v0 = '0.1.0-1'
        v1 = '0.1.1-1'

        self._Install(p, v0)
        self.clear_signals()

        self._RefreshPackageCache("", "")
        self.clear_signals()

        err = 'RequiresSystemUpgrade'
        self.failUnlessRaisesDBusException(err, self._Upgrade, p, v1)
        self.check_failed_upgrade(p, v1, None, dbus_error(err))
        P = self._FetchPackageData(p, v1)
        self.assertEqual(P['ErrorName'], dbus_error(err))

        up = P['DependsSystemUpgradeName']
        uv = P['DependsSystemUpgradeVersion']
        self._download_system_upgrade(up, uv)
        self._upgrade_system(up, uv)

        # The error should remain
        P = self._FetchPackageData(p, v1)
        self.assertEqual(P['ErrorName'], dbus_error(err))

    def test_install_upgrade_uninstall_user_system_ok(self):
        # Install/upgrade/uninstall user/system package that is not
        # system metapackage
        p = 'hpm-user-system'
        v0 = '0.1.0-1'
        v1 = '0.1.1-1'

        self._RefreshPackageCache("", "")
        self.clear_signals()

        self._Install(p, v0)
        self.check_install(p, v0)
        self.clear_signals()

        self._Upgrade(p, v1)
        self.check_upgrade(p, v1)
        self.clear_signals()

        self._Uninstall(p)
        self.clear_signals()

    def test_Upgrade_fail_system_package(self):
        p = 'hpm-fake-system-upgrade'
        v0 = '0.1.0-1'
        v1 = '0.1.1-1'

        self._Install(p, v0)
        self.check_install(p, v0)
        self.clear_signals()

        P = self._FetchPackageData(p, v0)
        self.assertEqual( P['Name'], p )
        self.assertEqual( P['Version'], v0 )
        self.check_key_not_in_dict(P, 'FlagSystemUpdate')

        self._RefreshPackageCache("", "")
        self.check_refresh_cache()
        self.clear_signals()

        P = self._FetchAvailableUpdates('')
        for pkg in P:
            self.failIf( pkg['Name'] == p )

        err = 'PackageNotFound'
        self.failUnlessRaisesDBusException(err,self._FetchPackageData, p, v1)

        err = 'PackageIsSystemPackage'
        self.failUnlessRaisesDBusException(err, self._Upgrade, p, v1)
        self.clear_signals()

        self._Uninstall(p)
        self.check_uninstall(p)
        self.clear_signals()

    def test_Upgrade_fail_depends_system_package(self):
        p = 'hpm-user-upgrade-depends-fake-system-pkg'
        v0 = '0.1.0-1'
        v1 = '0.1.1-1'

        self._Install(p, v0)
        self.check_install(p, v0)
        self.clear_signals()

        self._RefreshPackageCache("", "")
        self.check_refresh_cache()
        self.clear_signals()

        err = 'RequiresSystemPackage'
        self.failUnlessRaisesDBusException(err, self._Upgrade, p, v1)
        self.clear_signals()

        self._Uninstall(p)
        self.check_uninstall(p)
        self.clear_signals()

    def _install_without_updates(self, p, v):
        self.downgrade_testrepo()
        self._RefreshPackageCache("", "")
        self.clear_signals()

        # Depends on hpm-user-dep2 (= 0.1.0-1)
        self._Install(p, v)
        self.clear_signals()

        system("update-testrepo")
        self._RefreshPackageCache("", "")
        self.clear_signals()

    def test_Upgrade_fails_as_it_would_remove_user_package(self):
        v0 = '0.1.0-1'
        v1 = '0.1.1-1'

        # Depends on hpm-user-dep2 (= 0.1.0-1)
        self._install_without_updates('hpm-user-old-deps', v0)
        self._Install('hpm-user-valid-deps4', v0)
        self.clear_signals()

        # Depends on hpm-user-dep2 (= 0.1.1-1)
        err = 'DependenciesPreventOperation'
        self.failUnlessRaisesDBusException(err, self._Upgrade, 'hpm-user-valid-deps4', v1)
        self._Uninstall('hpm-user-old-deps')
        self.clear_signals()

        # If conflicts or breaks/replaces, no error
        # See SWP#PkMgr-557
        self._Install('hpm-user-upgrade-conflicts-hpm-user-old-deps', v0)
        self._install_without_updates('hpm-user-old-deps', v0)
        self._Upgrade('hpm-user-upgrade-conflicts-hpm-user-old-deps', v1)
        self.clear_signals()
        self.check_package_not_installed('hpm-user-old-deps')
        self._Uninstall('hpm-user-upgrade-conflicts-hpm-user-old-deps')
        self.clear_signals()
        self.check_package_not_installed('hpm-user-old-deps')

        self._Install('hpm-user-upgrade-breaks-hpm-user-old-deps', v0)
        self._install_without_updates('hpm-user-old-deps', v0)
        self._Upgrade('hpm-user-upgrade-breaks-hpm-user-old-deps', v1)
        self.clear_signals()
        self.check_package_not_installed('hpm-user-old-deps')
        self._Uninstall('hpm-user-upgrade-breaks-hpm-user-old-deps')
        self.clear_signals()

    def test_Upgrade_fails_as_it_would_downgrade_package(self):
        p = 'hpm-user-upgrade-downgrades-hpm-user-dep1'
        v0 = '0.1.0-1'
        v1 = '0.1.1-1'

        system("update-testrepo")
        self._RefreshPackageCache("", "")
        self.clear_signals()

        self._Install(p, v0)

        err = 'NeedDowngrade'
        self.failUnlessRaisesDBusException(err, self._Upgrade, p, v1)
