# -*- python -*-
#
# Copyright (C) 2009 - 2011 Nokia Corporation.
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License version
# 2 as published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see http://www.gnu.org/licenses/.


import unittest
import dbus
import commands
import sys
import datetime

from test_helper import *

def pkg_filename(path, name, version='0.1.0-1', arch=deb_arch()):
    return '%s/%s_%s_%s.deb' % (path, name, version, arch)

def dbus_error(str):
    return 'com.nokia.package_manager.Error.'+str

class test_daemon_dbus_upgrade_all(tc_pkgmgrd_helper):
    def setUp(self):
        self.ignoredlist = '/var/lib/package-manager/ignored-packages'
        tc_pkgmgrd_helper.setUp(self)
#        setup_testrepo_upgrade_packages()
        pkgdir = get_pkgdir()
        arch = deb_arch()

        for i in range(3):
            n = i+1
            system("pkgmgr uninstall -p hpm-user%d" % n, False)
            system("pkgmgr install -p hpm-user%d -v %s" % (n, '0.1.0-1'))

    def tearDown(self):
        tc_pkgmgrd_helper.tearDown(self)

    def test_add_package_to_ignore_list(self):
        p = 'hpm-user2'
        v = '0.1.1-1'
        
        # Make sure that there is no ignored-packages
        if ( os.path.exists( self.ignoredlist ) ):
            os.remove( self.ignoredlist )
            
        self._ignore_update_package(p, v)

        self.failUnless(os.path.exists( self.ignoredlist ))

        f = open( self.ignoredlist, "r")
        text = f.read()
        f.close()
        tokens = text.split()
        self.failUnless( tokens[0] == p )
        self.failUnless( tokens[1] == ('v=%s') % v )

        # Check also that refresh cache's clear ignored flag clears it.
        self._RefreshPackageCache("", "clearignored") # Clear the ignored list
        self.clear_signals()

        f = open( self.ignoredlist, "r")
        text = f.read()
        f.close()
        self.assertEqual(text, "")
        
    def test_upgrade_all_with_os(self):
        # Test upgrading all packages.  One requires a OS update.

        self._Install ('hpm-user-depends-system-upgrade', '0.1.0-1')
        self.clear_signals()

        setup_testrepo_upgrade_packages()
        self._RefreshPackageCache("", "")
        self.clear_signals()

        updates = map (lambda u: u['Name'], self._FetchAvailableUpdates(''))

        # start the operation
        self.reply = None
        self.error = None
        self.iface.upgrade_all(False,
                               reply_handler=self._async_reply,
                               error_handler=self._async_error,
                               timeout=self.tmo)

        # wait for the operation_started signal
        s = self._wait_signal (60)

        op = self._fetch_current_operation_data ()

        # check that all pending packages are also in 'updates'
        for pv in op['PendingPackages']:
            [p, v] = pv.split ('=')
            self.failUnless (p in updates,
                             'Package %s is going to be updated but was not listed' % p)

        self.failUnless ('RejectedPackages' in op)

        rej = { }
        for pve in op['RejectedPackages']:
            [p, v, e] = pve.split ('=')
            rej[p] = { 'v': v, 'e': e }

        p = 'hpm-user-upgrade-missing-deps'
        self.failUnless (p in rej)
        self.failUnless (rej[p]['v'] == '0.1.1-1')
        self.failUnless (rej[p]['e'] == 'com.nokia.package_manager.Error.MissingDependencies')

        p = 'hpm-user-depends-system-upgrade'
        self.failUnless (p in rej)
        self.failUnless (rej[p]['v'] == '0.1.1-1')
        self.failUnless (rej[p]['e'] == 'com.nokia.package_manager.Error.RequiresSystemUpgrade')

        # wait for the operation_complete signal
        self.loop.run()
        self.clear_signals()
