#
# Copyright (C) 2009 - 2011 Nokia Corporation.
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License version
# 2 as published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see http://www.gnu.org/licenses/.

import datetime
import unittest
import commands
import glob
import re
import sys
import dbus
import os.path
import os
import fnmatch
import shutil
from testconf import host_arch
from xml.dom.minidom import parse, parseString

def in_scratchbox():
    return os.getenv("_SBOX_DIR")

def set_environment():
    os.environ["no_proxy"] = "127.0.0.1"
    if not os.getenv("_SBOX_DIR"):
        os.unsetenv("AEGIS_FIXED_ORIGIN")

def get_homedir():
    if os.path.exists('/home/user'):
        return '/home/user'
    else:
        return os.getenv('HOME')

def get_testdir():
    return '/var/lib/package-manager-testrepo'

def get_repodir():
    return get_testdir()+'/'+'testrepo'

def get_pkgdir():
    return get_testdir()+'/'+'testrepo/pool/user/development'
    
def get_reboot_script():
    return '/usr/lib/package-manager/reboot.sh'

def get_local_pkgdir():
    if os.path.exists("./testpackages"):
        return os.getcwd() + "/testpackages";
    else:
        return "/usr/share/package-manager-tests";

def dbus_error(str):
    return 'com.nokia.package_manager.Error.'+str

def get_pm_bar_conf():
    return '/usr/share/backup-framework/applications/package-manager-backup.conf'

def get_backup_dir():
    return '%s/.package-manager/' % get_homedir()

def get_backup_src_list_dir():
    return os.path.join(get_backup_dir(), 'sources-lists')

def get_backup_src_lists_dir():
    return os.path.join(get_backup_src_list_dir(), 'sources.list.d')

def get_backup_file():
    return '%s/.package-manager/installed-packages.backup' % get_homedir()

def get_restore_file():
    return '/var/lib/package-manager/installed-packages.restored'

def get_restore_file_tmp():
    return '%s/.package-manager/installed-packages.restored' % get_homedir()

def get_backup_script():
    return '/usr/share/package-manager/package-manager-backup'
    
def get_restore_script():
    return '/usr/share/package-manager/package-manager-restore'
    
def get_sources_list_dir():
    return '/etc/apt/sources.list.d/'

def install_file(src, dest):
    if os.path.exists(src) and not os.path.exists(dest):
        shutil.copy(src, dest)

# Setup backup & restore scripts and directories in case
# package-manager package is not installed.
def setup_bar_files():
    if not os.path.exists(get_backup_src_list_dir()):
        os.makedirs(get_backup_src_list_dir())
    if not os.path.exists(get_backup_src_lists_dir()):
        os.makedirs(get_backup_src_lists_dir())
    install_file('../daemon/package-manager-backup', get_backup_script())
    install_file('../daemon/package-manager-restore', get_restore_script())

def now():
    d = datetime.datetime.now()
    # Strip microseconds
    return datetime.datetime(d.year, d.month, d.day, d.hour, d.minute, d.second, 0)

# pkgmgr-test-" + deb_arch()[1]
#    ret = commands.getstatusoutput("source ./testvars.sh; echo -n $TESTDIR")
#    if ret[0] != 0:
#        sys.exit("Failed get testdir: %s" % ret[1])
#    return ret[1]

user_categories = (
    'user/desktop',
    'user/development',
    'user/education',
    'user/games',
    'user/graphics',
    'user/multimedia',
    'user/navigation',
    'user/network',
    'user/office',
    'user/science',
#    'user/system',
    'user/utilities'
    )

script_dir = '.'
invalid_arch = 'powerpc'
user_comp = 'user/development'
repo_name = 'testrepo'
nonuser_comp = 'misc'
install_ver = ['0.1.0']
upgrade_ver = ['0.1.0', '0.1.1']
install_pkgs = [
    {'name': 'hpm-user-invalid-arch',
     'arch': invalid_arch},
    {'name': 'hpm-user-missing-deps',
     'deps': 'hpm-nonexisting'},
    {'name': 'hpm-nonuser',
     'comps': [nonuser_comp]},
    {'name': 'hpm-nonuser-dep',
     'versions': upgrade_ver,
     'comps': [nonuser_comp]},
    {'name': 'hpm-user-valid-deps',
     'versions': upgrade_ver,
     'deps': 'hpm-user-dep1, hpm-user-dep2, hpm-user-dep3, hpm-user-dep4, hpm-user-dep5, hpm-nonuser-dep'},
    {'name': 'hpm-user-files',
     'files': True},
    {'name': 'hpm-user-failed-dep',
     'deps': 'hpm-user-preinst-fail'}
    ]

i = 0
for c in user_categories:
    i += 1
    install_pkgs.append(
        {'name': 'hpm-user%d' % i,
         'comps': [c],
         'versions': upgrade_ver},
        )

for i in range(10):
    install_pkgs.append(
        {'name': 'hpm-user-dep%d' % (i+1),
         'versions': upgrade_ver},
        )

upgrade_pkgs = (
    {'name': 'hpm-user-upgrade-no-upgrade'},
    {'name': 'hpm-user-upgrade1',
     'versions': upgrade_ver},
    {'name': 'hpm-user-upgrade2',
     'versions': upgrade_ver},
    {'name': 'hpm-user-upgrade-dep',
     'versions': upgrade_ver},
    {'name': 'hpm-user-upgrade-dep1',
     'versions': upgrade_ver},
    {'name': 'hpm-user-upgrade-dep2',
     'versions': upgrade_ver},
    {'name': 'hpm-user-upgrade-dep3',
     'versions': upgrade_ver},
    {'name': 'hpm-user-upgrade-dep4',
     'versions': upgrade_ver},
    {'name': 'hpm-user-upgrade-dep5',
     'versions': upgrade_ver},
    {'name': 'hpm-nonuser-upgrade',
     'versions': upgrade_ver,
     'comps': [nonuser_comp]},
    {'name': 'hpm-nonuser-upgrade-dep',
     'versions': upgrade_ver,
     'comps': [nonuser_comp]},
    {'name': 'hpm-user-upgrade-missing-deps',
     'versions': install_ver,
     'deps': 'hpm-user-upgrade-dep, hpm-nonuser-upgrade-dep'},
    {'name': 'hpm-user-upgrade-missing-deps',
     'versions': ['0.1.1'],
     'deps': 'hpm-user-upgrade-dep, hpm-nonuser-upgrade-dep, hpm-nonexisting'},
    {'name': 'hpm-user-upgrade-valid-deps',
     'versions': upgrade_ver,
     'deps': 'hpm-user-upgrade-dep1, hpm-user-upgrade-dep2, hpm-user-upgrade-dep3, hpm-user-upgrade-dep4, hpm-user-upgrade-dep5, hpm-nonuser-dep'},
    {'name': 'hpm-long-install',
     'delays': [20, 20]},
    )

system_pkgs = [
    {'name': 'hpm-system-meta',
     'versions': ['0.1.0'],
     'comps': ['user/system'],
     'deps': 'hpm-system1 (= 0.1.0-1), hpm-system2 (= 0.1.0-1), hpm-system3 (= 0.1.0-1)'},
    {'name': 'hpm-system-meta',
     'versions': ['0.1.1'],
     'comps': ['user/system'],
     'deps': 'hpm-system1 (= 0.1.1-1), hpm-system2 (= 0.1.1-1), hpm-system3 (= 0.1.1-1)'},
    ]
for i in range(3):
    system_pkgs.append(
        {'name': 'hpm-system%d' % (i+1),
         'comps': ['misc'],
         'versions': upgrade_ver},
        )

pkgs = []
pkgs.extend(install_pkgs)
pkgs.extend(upgrade_pkgs)
pkgs.extend(system_pkgs)

def get_repos():
    test_dir = get_testdir()
    repos = (
        {
            'name': repo_name,
            'dir': test_dir+'/'+repo_name,
            'dist': 'default',
            'comps': ['user', 'main']
            },
#     {
#         'name': 'TestEssential',
#         'dir': '%s/%s' % (test_dir, 'TestEssential'),
#         'dist': 'default',
#         'comps': ['user', 'main'],
#         'essential': True
#         },
        )
    return repos

def make_path():
    if os.path.isdir('../daemon'):
        d = '..'
        return "PATH=%s:%s/test:%s/test/testrepo:%s/cli:%s/daemon:$PATH" % (script_dir, d, d, d, d)
    else:
        d = os.path.abspath('.')
        return "PATH=%s:/usr/share/package-manager-testrepo:$PATH" % d

def system(c, exit_on_error=True):
    cc = "%s %s </dev/null" % (make_path(), c)

    ret = commands.getstatusoutput(cc)
    if exit_on_error and ret[0] != 0:
        sys.exit("Error: Command '%s' failed: %s" % (cc, ret[1]))
    else:
        l = list(ret)
        l[0] = l[0] >> 8
        return l

def system_sbox(c, exit_on_error=True):
    sbox_preload = os.getenv('SBOX_PRELOAD')
    if sbox_preload:
        cc = "%s %s %s </dev/null" % (make_path(), 'LD_PRELOAD=%s' % sbox_preload, c)
    else:
        cc = "%s %s </dev/null" % (make_path(), c)

#    cc = "%s LD_PRELOAD=$SBOX_PRELOAD %s </dev/null" % (make_path(), c)
#    cc = "%s %s </dev/null" % (make_path(), c)
#    cc = "%s %s </dev/null" % (make_path(), c)
#    print c
    ret = commands.getstatusoutput(cc)
    if exit_on_error and ret[0] != 0:
        sys.exit("Error: Command '%s' failed: %s" % (cc, ret[1]))
    else:
        l = list(ret)
        l[0] = l[0] >> 8
        return l

def deb_arch():
    """Detect target architecture"""
    return host_arch

def start_pkgmgrd():
    ret = system("start-pkgmgrd")
#    print "start_pkgmgrd: "+ret[1]
    return ret

def stop_pkgmgrd():
    ret = system('stop-pkgmgrd')
    if ret[0] != 0:
        sys.exit("Failed to stop pkgmgrd: %s" % ret[1])
#    print "stop_pkgmgrd: "+ret[1]
    stop_dbus_thread()

def get_sources_list_files():
    F = glob.glob('/etc/apt/sources.list')
    F.extend(glob.glob('/etc/apt/sources.list.d/*.list'))
    l = '/etc/apt/sources.list.d/testrepo.list'
    if l in F:
        F.remove(l)
    return F

def disable_repositories():
    F = get_sources_list_files()
    for f in F:
#        print f
        os.rename(f, '%s.disabled' % f)

def enable_repositories():
    F = glob.glob('/etc/apt/sources.list.disabled')
    F.extend(glob.glob('/etc/apt/sources.list.d/*.list.disabled'))

    for f in F:
        n = re.search(r'(^.*)\.disabled$', f).group(1)
#        print f, n
        os.rename(f, n)

def remove(f):
    if os.path.exists(f):
        os.remove(f)

def empty_dir(folder):
    ok = True
    for f in os.listdir(folder):
        file_path = os.path.join(folder, f)
        try:
            if os.path.isfile(file_path):
                os.remove(file_path)
            if os.path.isdir(file_path):
                if not empty_dir(file_path):
                    ok = False
                os.rmdir(file_path)
        except Exception, e:
            print e
            ok = False
    return ok

package_manager_config = {
    'disableReboot': 0,
    'assumeNetwork': 1
}

def store_package_manager_config():
    if (os.path.exists('/etc/package-manager/config')
        and not os.path.exists('/etc/package-manager/config.orig')):
        os.rename('/etc/package-manager/config',
                  '/etc/package-manager/config.orig')

def restore_package_manager_config():
    if (os.path.exists('/etc/package-manager/config.orig')):
        os.rename('/etc/package-manager/config.orig',
                  '/etc/package-manager/config')

def write_package_manager_config():
    if not os.path.exists('/etc/package-manager'):
        os.makedirs('/etc/package-manager')
    f = open('/etc/package-manager/config', 'w')
    f.write('[Main]\n')
    for k, v in package_manager_config.items():
      f.write('%s=%s\n' % (k, v))
    f.close()

def disable_reboot_script():
    package_manager_config['disableReboot'] = 1
    write_package_manager_config()

    # Sbox: use reboot.sh from source tree if we're in
    # there. Rationale: tests should work without installing packages
    # locally.
    p = '/usr/lib/package-manager'
    if not os.path.exists(p):
        os.makedirs(p)
    f = '../daemon/reboot.sh'
    if os.path.exists(f):
        # Store copy of original reboot.sh
        orig = get_reboot_script()
        orig_disabled = '%s.disabled' % orig
        if os.path.exists(orig):
            shutil.copy(orig, orig_disabled)

        # Replace it by one from source tree
        shutil.copy(f, p)

    if ( os.path.exists('/var/tmp/pkgmgrd.reboot') ):
        os.remove('/var/tmp/pkgmgrd.reboot')

def enable_reboot_script():
    package_manager_config['disableReboot'] = 0
    write_package_manager_config()

    # Restore original reboot.sh
    orig = get_reboot_script()
    orig_disabled = '%s.disabled' % orig
    if os.path.exists(orig_disabled):
        remove(orig)
        os.rename(orig_disabled, orig)

    if ( os.path.exists('/var/tmp/pkgmgrd.reboot') ):
        os.remove('/var/tmp/pkgmgrd.reboot')

def install_packages():
#    inst_pkgs = [
#        'hpm-user-upgrade1=0.1.0-1',
#        'hpm-user-upgrade2=0.1.0-1',
#        'hpm-user-upgrade-dep=0.1.0-1',
#        'hpm-nonuser-upgrade-dep=0.1.0-1',
#        'hpm-user-upgrade-missing-deps=0.1.0-1',
#        'hpm-user-upgrade-no-upgrade=0.1.0-1'
#        ]

    I = ['hpm-user-upgrade1',
         'hpm-user-upgrade-valid-deps',
         'hpm-user-upgrade-missing-deps',
         'hpm-user-upgrade-dep',
         'hpm-nonuser-upgrade-dep',
         'hpm-nonuser-upgrade']
    for i in range(5):
        I.append('hpm-user-upgrade-dep%d' % (i+1))

    Iv = []
    v = '0.1.0-1'
    for p in I:
        Iv.append('%s=%s' % (p, v))
#    Iv.append('hpm-user-upgrade2=0.1.1-1')

    Iv.append('hpm-system-meta=0.1.0-1')
    Iv.append('hpm-system1=0.1.0-1')
    Iv.append('hpm-system2=0.1.0-1')
    Iv.append('hpm-system3=0.1.0-1')
    Iv.append('hpm-user-failed-dep-')

    system("apt-get.real -y --force-yes update")
    system("apt-get.real -y --force-yes install %s" % ' '.join(Iv))

def setup_testrepo_upgrade_packages():
    system('update-testrepo')
    system('pkgmgr list-updates -r')

def clear_test_packages_in_apt_archives_dir():
    system('rm /var/cache/apt/archives/hpm-*.deb', False)
    system('rm /var/cache/apt/archives/partial/hpm-*.deb', False)

def init_tests():
#    pass
#    system("testrepo/repo-install.sh testrepo/testrepo_2.tar.gz")
#    system("./setup-testrepo.sh")
    set_environment()
    stop_pkgmgrd()
    disable_repositories()
    system("setup-testrepo")
    install_packages()
#    start_pkgmgrd()

def uninit_tests():
    stop_pkgmgrd()
    enable_repositories()

def apt_get_update():
    ret = commands.getstatusoutput("apt-get update </dev/null")
    if ret[0] != 0:
        sys.exit("apt-get update failed:\n%s" % ret[1])

def is_package_installed(name, want_true):
    ret = commands.getstatusoutput('dpkg -s %s | grep "^Status.* installed$"' % name)
#    if (ret[0] != 0 and want_true) or (ret[0] == 0 and not want_true):
#        print "is_package_installed(%s):\n'%s'" % (name, ret[1])
    return want_true ^ (ret[0] != 0)

def is_package_installed_version(name, version):
    if is_package_installed(name, True):
        ret=commands.getstatusoutput('dpkg -s %s | grep "^Version: %s$"' % (name, version))
        return ret[0] == 0
    else:
        return False

def get_user_category(section_line):
    cat = section_line[len("Section: "):]
    if cat =="user/desktop":
        return "Desktop";
    elif cat =="user/development":
        return "Programming";
    elif cat =="user/education":
        return "Education";
    elif cat =="user/games":
        return "Games";
    elif cat =="user/graphics":
        return "Graphics";
    elif cat =="user/multimedia":
        return "Multimedia";
    elif cat =="user/navigation":
        return "Navigation";
    elif cat =="user/network":
        return "Internet & Networking";
    elif cat =="user/office":
        return "Office";
    elif cat =="user/science":
        return "Science";
    elif cat =="user/system":
        return "System";
    elif cat =="user/utilities":
        return "Accessories";
    else:
        return "Other";

def is_package_available(info):
    lines = info.split("\n")
    for line in lines:
        if "Filename:" in line:
            return True
    return False

def is_user_package(info, category=None):
    lines = info.split("\n")
    for line in lines:
        if "Section: " in line:
            if "user/" in line:
                if category is None:
                    return True
                else:
                    cat = get_user_category(line)
                    return category == cat
            return False
    return False

def is_package_available_and_user(name, category=None):
    ret=commands.getstatusoutput('apt-cache show %s' % name)
    if ret[0] != 0:
        return False
    return is_user_package(ret[1], category) and is_package_available(ret[1])

def is_package_available_in_repo(name, category=None):
    ret=commands.getstatusoutput('apt-cache show %s' % name)
    if ret[0] != 0:
        return False
    return is_package_available(ret[1])

def get_all_packages_list():
    ret=commands.getstatusoutput('apt-cache pkgnames')
    if ret[0] != 0:
        return None
    return ret[1].split("\n")

def get_available_packages_list(category=None):
    all = get_all_packages_list()
    available = []
    for pkg in all:
#        print "Checking package: %s" % pkg
        if is_package_available_and_user(pkg, category) and not is_package_installed(pkg):
            available.append(pkg)
    return available

def get_installed_packages_list():
    ret=commands.getstatusoutput("dpkg-query --showformat '${Package}\t${Version}\t${Section}\t${Status}\n' -W | grep '.*\t.*\tuser/.*\t.*'")
    installed = []
    lines = ret[1].split("\n")
    for line in lines:
        if "install ok installed" in line:
            t = line.split("\t")
            installed.append((t[0], t[1]))
    return installed

def catalogue_exists(iface, name):
    cats = iface.FetchCatalogues()
    results = []
    for cat in cats:
        results.append(cat['Name'])
    return name in results

def find_pkg_from_array(array, name):
    for p in array:
        if p['Name'] == name:
            return p

namespace='com.nokia.package_manager'

import dbus.glib
import gobject
import threading
import time

gobject.threads_init()

from dbus.mainloop.glib import DBusGMainLoop
DBusGMainLoop(set_as_default=True)
dbus.mainloop.glib.threads_init()

class DBusThread(threading.Thread):
    def __init__ (self):
        threading.Thread.__init__(self)
        self.loop = None

    def stop(self):
        if self.loop:
            self.loop.quit()

    def run(self):
        self.loop = gobject.MainLoop()
        self.loop.run()
        self.loop = None

global __dbus_thread
__dbus_thread = DBusThread()
global signals
signals = None

def get_instance():
    global __dbus_thread
    return __dbus_thread

def stop_dbus_thread():
    global __dbus_thread
    if __dbus_thread.isAlive():
#        print "stop_dbus_thread()"
        __dbus_thread.stop()
        __dbus_thread.join(None)
        while __dbus_thread.loop != None:
            time.sleep(0.1)

    __dbus_thread = DBusThread()

def start_dbus_thread():
#    print "start_dbus_thread()"
    global __dbus_thread
    __dbus_thread.start()

    while __dbus_thread.loop == None:
        time.sleep(0.1)
    return __dbus_thread.loop

dump_signals=False

def download_progress(operation, packagename, ver, already, total):
    if dump_signals:
        print "download_progress(%s, %s, %s, %d, %d)" % (operation, packagename, ver, already, total)
    global signals
    if signals is None:
        print "self.signals is None"
        signals = []
    signals.append(("download_progress", operation, packagename, ver, already, total))

def opStarted(op, pkg, ver):
    if dump_signals:
        print "Operation started: %s, %s, %s" % (op, pkg, ver)
    global signals
    if signals is None:
        print "self.signals is None"
        signals = []
    signals.append(("operation_started", op, pkg, ver))

def opProgress(op, pkg, ver, percentage):
    if dump_signals:
        print "OperationProgress: %s, %s, %s, %d %%" % (op, pkg, ver, percentage)
    global signals
    if signals is None:
        print "self.signals is None"
        signals = []
    signals.append(("operation_progress", op, pkg, ver, percentage))

def opComplete(op, pkg, ver, error, reboot):
    if dump_signals:
        print "opComplete(%s, %s, %s, %s, %s)" % (op, pkg, ver, error, str(reboot))
    global signals
    if signals is None:
        print "self.signals is None"
        signals = []
    signals.append(("operation_complete", op, pkg, ver, error, reboot))

def pkgListUpdated(changes):
    if dump_signals:
        print "pkgListUpdated(%s)" % changes
    global signals
    if signals is None:
        print "self.signals is None"
        signals = []
    signals.append(("package_list_updated", changes))

def pkgIgnored(ign_type, pkg, ver):
    if dump_signals:
        print "pkgIgnored(%s, %s, %s)" % (ign_type, pkg, ver)
    global signals
    if signals is None:
        print "self.signals is None"
        signals = []
    signals.append(("package_ignored", ign_type, pkg, ver))

def pkg_filename(path, name, version='0.1.0-1', arch=deb_arch()):
    return '%s/%s_%s_%s.deb' % (path, name, version, arch)

class tc_pkgmgrd_helper(unittest.TestCase):
    def setup_dbus_signals(self):
        self.sysbus = dbus.SystemBus()
        self.obj = self.sysbus.get_object(namespace,'/com/nokia/package_manager')
        self.iface = dbus.Interface(self.obj, dbus_interface='com.nokia.package_manager')
        iface = dbus.Interface(self.obj, dbus_interface='com.nokia.package_manager')
        self.setup_signals(iface)
        self.loop = gobject.MainLoop()

    def setUp(self):
#        disable_repositories()
        self.timestamp_storage_file = '/var/lib/package-manager/installed-packages'
        remove(self.timestamp_storage_file)
        self.available_timestamp_file = '/var/lib/package-manager/available-packages'
        remove(self.available_timestamp_file)
        self.cache_refreshed_file = '/var/lib/package-manager/cacherefreshed'
        remove(self.cache_refreshed_file)

        remove('/var/lib/package-manager/failed-installs')
        remove('/var/lib/package-manager/failed-upgrades')

        open('/etc/testrepo/preinst.status', 'w').write('1\n')
        open('/etc/testrepo/postinst.status', 'w').write('0\n')

        store_package_manager_config()
        write_package_manager_config()

        start_pkgmgrd()

        self.setup_dbus_signals()

        self.category = ''
        self.filename_ok = ''

        self.tmo = 3*60*1000

    def tearDown(self):
        remove(self.cache_refreshed_file)

        global signals
        if os.path.exists(self.timestamp_storage_file):
            os.remove(self.timestamp_storage_file)
#        enable_repositories()
        stop_dbus_thread()
        restore_package_manager_config()
        signals = None
        self.download_progress.remove()
        self.op_started.remove()
        self.op_progress.remove()
        self.op_complete.remove()
        self.pkg_list_updated.remove()
        self.iface = None
        self.obj = None
        self.sysbus = None
        self.loop = None

    def failUnlessRaisesDBusException(self, excStr, callableObj, *args, **kwargs):
        fullExcStr = 'com.nokia.package_manager.Error.%s' % excStr
        try:
            callableObj(*args, **kwargs)
        except dbus.DBusException, e:
            if e.get_dbus_name() == fullExcStr:
                return
            else:
                errStr = "%s(%s): Wanted %s, got %s. Message: %s" % (str(callableObj), str(args), fullExcStr, e.get_dbus_name(), e.get_dbus_message())
                raise self.failureException, errStr
        else:
            raise self.failureException, "%s(%s): DBusException not raised" % (str(callableObj), str(args))

    def get_signals(self):
        global signals
        return signals

    def setup_signals(self, iface):
        global signals

        if signals is None:
            self.download_progress = iface.connect_to_signal("download_progress", download_progress)
            self.op_started = iface.connect_to_signal("operation_started", opStarted)
            self.op_progress = iface.connect_to_signal("operation_progress", opProgress)
            self.op_complete = iface.connect_to_signal("operation_complete", opComplete)
            self.pkg_list_updated = iface.connect_to_signal("package_list_updated", pkgListUpdated)
            self.pkg_ignored = iface.connect_to_signal("package_ignored", pkgIgnored)
#            start_dbus_thread()
        if signals:
            print "Got uncleared signals: %s" % str(signals)
        self.clear_signals()

    def clear_signals(self):
        global signals
        signals  = []

    def downgrade_testrepo(self):
        system("downgrade-testrepo", True)

    def _async_reply(self):
#        print "async_reply"
        self.loop.quit()

    def _async_error(self, e):
#        print "async_error"
        self.error = e
        self.loop.quit()

    def _started_signal_handler(self, op, pkg, version):
        self.operation_started_signal = (op, pkg, version)
        self.loop.quit()

    def _signal_timeout(self):
        # print "timeout"
        self.loop.quit()
        return False

    def _wait_signal(self, timeout=0):
        self.operation_started_signal = None
        event_id = gobject.timeout_add(timeout*1000, self._signal_timeout)
        started = self.iface.connect_to_signal("operation_started",
                                                self._started_signal_handler)

        self.loop.run()
        started.remove()
        gobject.source_remove(event_id)
        if not self.operation_started_signal:
            self.fail('No signal received in %d seconds' % timeout)
        return self.operation_started_signal

    def _Install(self, p, v=''):
        self.reply = None
        self.error = None
        self.iface.install(p,
                           v,
                           reply_handler=self._async_reply,
                           error_handler=self._async_error,
                           timeout=self.tmo)
        self.loop.run()
        if self.error:
            raise self.error

    def _InstallFile(self, f):
        self.reply = None
        self.error = None
        self.iface.install_file(f,
                                reply_handler=self._async_reply,
                                error_handler=self._async_error,
                                timeout=self.tmo)
        self.loop.run()
        if self.error:
            raise self.error

    def _Upgrade(self, p, v=''):
        self.reply = None
        self.error = None
        self.iface.upgrade(p,
                           v,
                           reply_handler=self._async_reply,
                           error_handler=self._async_error,
                           timeout=self.tmo)
        self.loop.run()
        if self.error:
            raise self.error

    def _Restore(self, p, v):
        self.reply = None
        self.error = None
        self.iface.restore(p,
                           v,
                           reply_handler=self._async_reply,
                           error_handler=self._async_error,
                           timeout=self.tmo)
        self.loop.run()
        if self.error:
            raise self.error

    def _Uninstall(self, p):
        self.reply = None
        self.error = None
        self.iface.uninstall(p,
                             reply_handler=self._async_reply,
                             error_handler=self._async_error,
                             timeout=self.tmo)
        self.loop.run()
        if self.error:
            raise self.error

    def _download_system_upgrade(self, p, v=''):
        self.reply = None
        self.error = None
        self.iface.download_system_upgrade(p,
                                           v,
                                           reply_handler=self._async_reply,
                                           error_handler=self._async_error,
                                           timeout=self.tmo)
        self.loop.run()
        if self.error:
            raise self.error

    def _upgrade_system(self, p, v=''):
        self.reply = None
        self.error = None
        self.iface.upgrade_system(p,
                                  v,
                                  reply_handler=self._async_reply,
                                  error_handler=self._async_error,
                                  timeout=self.tmo)
        self.loop.run()
        if self.error:
            raise self.error

    def _upgrade_all(self):
        self.reply = None
        self.error = None
        self.iface.upgrade_all(False,
                               reply_handler=self._async_reply,
                               error_handler=self._async_error,
                               timeout=self.tmo)
        self.loop.run()
        if self.error:
            raise self.error
    
    def _ignore_update_package(self, p, v):
        self.iface.ignore_update_package(p, v)

    def _ignore_restore_package(self, p):
        self.iface.ignore_restore_package(p)

    def _RefreshPackageCache(self, domain, flags):
        self.reply = None
        self.error = None
        self.iface.refresh_package_cache(domain, flags,
                                         reply_handler=self._async_reply,
                                         error_handler=self._async_error,
                                         timeout=self.tmo)
        self.loop.run()
        if self.error:
            raise self.error

    def _fetch_current_operation_data(self):
        return self.iface.fetch_current_operation_data()
    
    def _FetchRestoreStatus(self):
        return self.iface.fetch_restore_status()
    
    def _FetchRestorablePackages(self):
        return self.iface.fetch_restorable_packages()

    def _SimulateOperation(self, op, p, v='', flags=''):
        return self.iface.simulate_operation(op, p, v, flags)

    def _FetchPackageData(self, p, v):
        return self.iface.fetch_package_data(p, v)

    def _FetchPackageDataInstalled(self, p):
        return self.iface.fetch_package_data_installed(p)

    def _FetchPackageDataFile(self, f):
        return self.iface.fetch_package_data_file(f)

    def _FetchPackageDependencyData(self, p, v=''):
        return self.iface.fetch_package_dependency_data(p, v)

    def _FetchFilesOfInstalledPackage(self, p):
        return self.iface.fetch_files_of_installed_package(p)

    def _FetchInstalled(self):
        return self.iface.fetch_installed()

    def _FetchPackages(self, category):
        return self.iface.fetch_packages(category)

    def _FetchAvailableUpdates(self, category):
        return self.iface.fetch_available_updates(category)

    def check_key_not_in_dict(self, dict_, key):
        if key in dict_:
            self.fail("'%s' not in dict, value == %s" % (key, dict_[key]))

    def check_dict_key_value(self, dict_, key, value):
        if key not in dict_:
            self.fail("'%s' in dict" % key)
        self.assertEqual(dict_[key], value)

    def check_pkg_ver_in_array(self, array, name, ver):
        p = find_pkg_from_array(array, name)
        if not p:
            self.fail("'%s' in array" % name)
        elif p['Version'] != ver:
            self.fail("'%s' in array, but found version '%s' != expected '%s'" % (name, p['Version'], ver))
        return p

    def check_packages_installed_or_available(self, names, installed):
        # Check with dpkg
        for n in names:
            self.failUnless(is_package_installed(n, installed))

        P = self._FetchPackages('')
        N = []
        for p in P:
            N.append(p['Name'])
        for n in names:
            if installed:
                self.failIf(n in N)
            else:
                self.failUnless(n in N)

        I = self._FetchInstalled()
        N = []
        for p in I:
            N.append(p['Name'])
        for n in names:
            if installed:
                self.failUnless(n in N)
            else:
                self.failIf(n in N)

    def package_has_timestamp(self, filename, package, version = None, pkgfilename = None):
        if not os.path.exists(filename):
            return False
        
        f = open(filename, 'r')
        lines = f.readlines()
        f.close()
        
        for l in lines:
            expected = "%s v=" % package
            if version is not None:
                expected = "%s%s" % (expected, version)
            expected = "%s [0-9]+" % expected
            if pkgfilename is not None:
                expected = "%s %s" % (expected, pkgfilename)
#            print expected
            if re.match(expected, l):
                return True
        return False

    def check_timestamp_file(self, filename, packages):
        self.failUnless( os.path.exists( filename ), "File does not exist. Filename: %s" %filename )
        
        f = open(filename, 'r')
        lines = f.readlines()
        f.close()

        length = len(packages)
        self.failUnless( length == len(lines), "Count of packages does not match the count of package timestamps" )
        
        cnt = 0
        found = False
        for pkg in packages:
            found = False
            for line in lines:
                expected = "%s v=%s" % (pkg[0], pkg[1])
                if ( expected in line ):
                    self.failIf( found, "There's two or more packages with a same name and version in the timestamp file. Package: %s" % pkg[0] )
                    cnt += 1
                    found = True
                    continue
            self.failUnless( found, "Package %s (%s) does not have a timestamp" % (pkg[0], pkg[1]) )
        self.failUnless(cnt == length, "There's either packages missing from timestamp file (%s) or there's too many packages in the file." % filename)

    def check_package_installed_or_available(self, name, version, installed, dont_check_timestamps = False, filename = None):
        # Check with dpkg
        self.failUnless(is_package_installed(name, installed))

        P = self._FetchPackages('')
        N = []
        for p in P:
            N.append(p['Name'])
        if installed:
            self.failIf(name in N)
        else:
            self.failUnless(name in N)

        I = self._FetchInstalled()
        N = []
        for p in I:
            N.append(p['Name'])
        if installed:
            self.failUnless(name in N)
        else:
            self.failIf(name in N)

        if dont_check_timestamps:
            return
        
        # Check the timestamps
        has_timestamp = self.package_has_timestamp(self.timestamp_storage_file, name, version, filename)
        self.failUnless( installed == has_timestamp )

    def check_package_installed(self, name):
        self.failUnless(is_package_installed(name, True),
                        "Package %s should be installed, but isn't" % name)
                        
    def check_package_available(self, name):
        self.failUnless(is_package_available_in_repo(name),
                        "Package %s should be available, but isn't" % name)

    def check_package_not_installed(self, name):
        self.failUnless(is_package_installed(name, False),
                        "Package %s should not be installed, but is" % name)
        
    def check_package_not_available(self, name):
        self.failUnless(not is_package_available_in_repo(name),
                        "Package %s should not be available, but is" % name)

    def check_package_installed_version(self, name, version):
        self.failUnless(is_package_installed_version(name, version),
                        "Package %s (%s) should be installed, but isn't" % (name, version))

    def check_package_not_installed_version(self, name, version):
        self.failIf(is_package_installed_version(name, version),
                    "Package %s (%s) should not be installed, but is" % (name, version))

    def check_uninstall(self, name, many_complete=False):
        self.check_package_not_installed(name)
        self.check_operation_started("Uninstall", name, "")
        self.check_operation_progress("Uninstall", name)
        self.check_operation_complete("Uninstall", name, None)
        self.check_signals_order("Uninstall", name, None, "", True, False, True, many_complete)

    def check_install(self, package, version, check_download=True, filename=None, error="", started=True, many_complete=False, allow_download=False, allow_incomplete_progress=False):
        op = 'Install'
        name = package
        ver = version
        self.check_package_installed(name)
        if filename is not None:
            name = filename
            ver = ''
            op = 'InstallFile'
        self.check_operation_started(op, name, ver)
        if check_download:
            self.check_download_progress(op, name)
        self.check_operation_progress(op, name, allow_incomplete = allow_incomplete_progress)
        self.check_operation_complete(op, name, None)
        self.check_signals_order(op, name, ver, error, started, check_download or allow_download, True, many_complete)
        self.failUnless( self.package_has_timestamp(self.timestamp_storage_file, package, version, filename) )

    def check_upgrade(self, name, version, check_version=True, many_complete=False):
        self.failUnless(is_package_installed_version(name, version))
        ver = version
        if ( check_version == False ):
            ver = ""
        self.check_operation_started("Upgrade", name, ver)
        self.check_download_progress("Upgrade", name)
        self.check_operation_progress("Upgrade", name)
        self.check_operation_complete("Upgrade", name, version)
        self.check_signals_order("Upgrade", name, ver, "", True, True, True, many_complete)
        self.failUnless( self.package_has_timestamp(self.timestamp_storage_file, name, version) )
    
    def check_restore(self, name, version, check_download=True, error="", started=True, many_complete=False):
        op = 'Restore'
        self.check_package_installed_version(name, version)
        self.check_operation_started(op, name, version)
        if check_download:
            self.check_download_progress(op, name)
        self.check_operation_progress(op, name)
        self.check_operation_complete(op, name, None)
        self.check_signals_order(op, name, version, error, started, check_download, True, many_complete)
    
    def check_refresh_cache(self, updates=True, changes=True):
        self.check_operation_started("Refresh", "", "")
#        self.check_package_list_updated(pkg_list_updated)
        self.check_package_list_updated(updates, changes)
        self.check_operation_complete("Refresh", "")
        self.check_signals_order("Refresh", "", "", "", True, False, False, False, updates)
        self.failUnless( os.path.exists(self.cache_refreshed_file) )

    def check_simulation_ok(self, result, download=True):
        self.assert_('Error' not in result )
        self.assert_('ErrorName' not in result )
        self.assert_('ErrorCode' not in result )
        self.assert_('RevertDeps' not in result )
        self.assert_('RevertUserDeps' not in result )

        try:
            result['DownloadSize']
        except KeyError:
            self.failIf(download, 'DownloadSize is missing')
        else:
            self.failIf(not download, 'There is a \'DownloadSize\' field even there should not.')
            if ( download ):
                self.failUnless( result['DownloadSize'] > 0 )
   
    def check_simulation_error(self, result, error, errorname, no_total_inst_sz=True, dont_check_revert_deps=False):
        self.assert_('ErrorName' in result )
        self.assert_('Error' in result )
        
        self.assert_( 'InstallDeps' not in result )
        self.assert_( 'InstallUserDeps' not in result )
        self.assert_( 'UpgradeDeps' not in result )
        self.assert_( 'UpgradeUserDeps' not in result )
        self.assert_( 'RemovePkgs' not in result )
        self.assert_( 'RemoveUserPkgs' not in result )

        self.assert_( 'DownloadSize' not in result )
        if ( no_total_inst_sz ):
            self.assert_( 'TotalInstalledSize' not in result )
        else:
            self.assert_( 'TotalInstalledSize' in result )
        
        self.assertEqual( errorname, result['ErrorName'] )
#        self.assertEqual( error, result['Error'] )
        if ( not dont_check_revert_deps ):
            self.assert_( 'RevertDeps' not in result )
            self.assert_( 'RevertUserDeps' not in result )


    def check_reboot(self, reboot_update=False):
        time.sleep(1) # Hack: race
        f = '/var/tmp/pkgmgrd.reboot'
        res = os.path.exists(f)
        self.failUnless( res )
        s = open(f, 'r').read()
        if reboot_update:
            self.assertEqual(s, "-u\n")
        else:
            self.assertEqual(s, "\n")

        if ( res ):
            os.remove(f)

    def check_failed_install(self, pkg, ver, fn, error_code, failed=True):
        f = '/var/lib/package-manager/failed-installs'
        fi = parse(f)
#        print fi.toxml()
        self.assertEqual(fi.documentElement.tagName, 'failed-installs')
        P = fi.getElementsByTagName('package')
        found = False
        for p in P:
            def text(p, tag):
                return p.getElementsByTagName(tag)[0].firstChild.data

            if text(p, 'name') == pkg:
                found = True
                self.assertEqual(text(p, 'name'), pkg)
                self.assertEqual(text(p, 'version'), ver)
                if fn:
                    self.assertEqual(text(p, 'filename'), fn)
                self.assertEqual(text(p, 'error-code'), error_code)

        if failed:
            self.assert_(found)
        else:
            self.assert_(not found)

    def check_failed_upgrade(self, pkg, ver, fn, error_code, failed=True):
        f = '/var/lib/package-manager/failed-upgrades'
        fi = parse(f)
#        print fi.toxml()
        self.assertEqual(fi.documentElement.tagName, 'failed-installs')
        P = fi.getElementsByTagName('package')
        found = False
        for p in P:
            def text(p, tag):
                return p.getElementsByTagName(tag)[0].firstChild.data

            if text(p, 'name') == pkg:
                found = True
                self.assertEqual(text(p, 'name'), pkg)
                self.assertEqual(text(p, 'version'), ver)
                if fn:
                    self.assertEqual(text(p, 'filename'), fn)
                self.assertEqual(text(p, 'error-code'), error_code)

        if failed:
            self.assert_(found)
        else:
            self.assert_(not found)

    def check_download_progress(self, operation, package):
#        print "check_download_progress operation: %s package %s" % (operation, package)
        global signals
        prevAlready = -1
        prevTotal = -1
        for sig in signals:
#            print "check_download_progress Signal: %s" % str(sig)
            if sig[0] == "download_progress":
                (name, op, pkg, ver, already, total) = sig
#                print "check_download_progress Signal is download_progress. Operation: %s Package: %s" % (op, pkg)
                self.assertEqual(pkg, package)#, "Wanted to progress for %s but got progress for %s", package, pkg)
                self.assertEqual(op, operation)
#                print "check_download_progress Downloaded %d of %d" % (already, total)
                self.failIf(already < 0)
                self.failIf(prevAlready >= already, "Downloading backwards or not downloading at all")
                self.failIf(already > total, "Downloaded more than total?")
                self.failIf(prevAlready == -1 and already != 0,"First download progress already downloaded was %d" % already)
                self.failIf(prevTotal > -1 and prevTotal != total, "Downloading amount is changing?")
                prevAlready = already
                prevTotal = total
        self.failIf(prevAlready == -1, "No progress info!")
        self.failIf(prevAlready != prevTotal, "Progress ended in %d of %d" % (prevAlready, prevTotal))

    def check_operation_started(self, operation, package, version):
#        print "check_operation_started() op: %s, pkg: %s, ver: %s" % (operation, package, version)
        global signals
#        print signals
        already = False
        for sig in signals:
#            print "check_operation_started() Signal: %s" % str(sig)
            if sig[0] == "operation_started":
                if ( already == True ):
                    self.fail("Operation started signal emited twice")
                (name, op, pkg, ver) = sig
#                print "check_operation_started() operation_started op: %s, pkg: %s, ver: %s" % (op, pkg, ver)
                self.assertEqual(pkg, package)#, "Wanted to progress for %s but got progress for %s", package, pkg)
                self.assertEqual(op, operation)
                if ( version != "" ):
                    self.assertEqual(ver, version)
                already = True
        self.failIf(already == False, "Operation started signal never emited")
    
    def check_no_operation_started(self):
#        print "check_no_operation_started()"
        global signals
#        print signals
        for sig in signals:
#            print "Signal: %s" % str(sig)
            if sig[0] == "operation_started":
                self.fail("Operation started signal emited even it should not!")

    def check_operation_progress(self, operation, package, version=None, allow_incomplete=False):
        global signals
        p = -1
        for sig in signals:
#            print "Signal: %s" % str(sig)
            if sig[0] == "operation_progress":
                (name, op, pkg, ver, percentage) = sig
                self.assertEqual(pkg, package)#, "Wanted to progress for %s but got progress for %s", package, pkg)
                self.assertEqual(op, operation)
                self.failIf(p==-1 and percentage != 0,"First operation progress percentage was %d" % percentage)
                self.failIf(p>=percentage, "Previous percentage (%d) was greater or equal next (%d)" % (p, percentage))
                p = percentage
        self.failIf(p == -1, "No progress info!")
        if not allow_incomplete:
            self.failIf(p != 100, "Progress ended in %d" % p)

    # Iterate thru all operation_complete signals and return a full
    # match or fail.
    def check_find_operation_complete(self, o, p, v, e=''):
        global signals
        for sig in signals:
            if sig[0] == "operation_complete":
                (name, op, pkg, ver, err, reboot) = sig
                if pkg == p:
                    if op != o:
                        self.fail('operation_complete of %s has operation=%s, %s expected' % (pkg, op, o))
                    if ver != v:
                        self.fail('operation_complete of %s has version=%s, %s expected' % (pkg, ver, v))
                    if err != e:
                        self.fail('operation_complete of %s has error=%s, %s expected' % (pkg, err, e))
                    return

        self.fail('no operation_complete for package %s' % p)

    def check_operation_complete(self, operation, package, version=None, error="", need_reboot=False):
        need_boot = False;
        global signals
        wait_for_signals = 5
        if error != "":
            error = "com.nokia.package_manager.Error." + error
#         print "Waiting for signal: %s" % str(("OperationComplete", operation, package, error))
        while wait_for_signals > 0:
            for sig in signals:
#                 print "Signal: %s" % str(sig)
                if sig[0] == "operation_complete":
                    (name, op, pkg, ver, err, reboot) = sig
#                    print "Reboot %s" % str(reboot)
                    if pkg == package and op == operation and err == error and reboot == need_reboot and (not version or (version and ver == version)):
#                     self.assertEqual(pkg, package)
#                     self.assertEqual(op, operation)
#                     self.assertEqual(err, error)
#                     print "Checked Signal: '%s' == '%s'" % (str(sig), (name, operation,package,error))
                        return
                    if reboot == True:
                      need_boot = True
                    if reboot == True and need_reboot == False:
                      self.fail("Operation wants to reboot the device even though it shouldn't!")
#                    if reboot == False and need_reboot == True:
#                      self.fail("Operation does not want to reboot the device even though it should!")
            wait_for_signals = wait_for_signals - 1
#            print "Waiting for OperationComplete signal..."
            time.sleep(1)
        if need_boot == False and need_reboot == True:
          self.fail("Operation does not want to reboot the device even though it should!")
        self.fail("No operation_complete received (waited for 5 seconds)!")
    
    def check_package_list_updated(self, pkg_list_updated, changes):
        global signals
        count = 0
        for sig in signals:
#            print "Signal: %s" % str(sig)
            if sig[0] == "package_list_updated":
                count += 1
                self.failUnless(sig[1] == changes)
        if ( pkg_list_updated ):
            self.failIf(count <= 0, "No package_list_updated signal!")
            self.failIf(count >= 2, "Too many package_list_updated signals!")
        else:
            self.failIf(count > 0, "There's a package_list_updated signal even though there shouldn't")

    def check_package_ignored(self, ign_type, pkg, ver):
        global signals
        count = 0
        for sig in signals:
#            print "Signal: %s" % str(sig)
            if sig[0] == "package_ignored":
                count += 1
                self.failUnless(sig[1] == ign_type)
                self.failUnless(sig[2] == pkg)
                self.failUnless(sig[3] == ver)
        self.failIf(count != 1, "Wrong amount of package_ignored signals")


# Checks that there are correct amount of signals and in correct order. Checks also basic parameters of the signals.
#    def check_signals_order(self, operation, package, version=None, error="", started=True, download=True, op_signal=True, many_complete=False, pkg_list=False):
    def check_signals_order(self, operation, package, version, error, started, download, op_signal, many_complete, pkg_list=False):
#        print "check_signals_order()"
        global signals
        s = signals[0]
        i = 0
        l = len( signals )
        
#        print ""
#        for iterator in range(l):
#            print signals[iterator]
#        print ""
        
#        print s
        if ( started == True ):
            self.failIf( s[0] != "operation_started", "Operation started signal is missing" )
            (name, op, pkg, ver) = s
            self.failIf(op != operation, "Wrong operation in operation started signal")
            self.failIf(pkg != package, "Wrong package name in operation started signal")
            if ( version != None and version != "" ):
                self.failIf( ver != version, "Wrong version in operation started signal")
            i += 1
        else:
            self.failIf( s[0] == "operation_started", "Operation started signal is emited even it shouldn't" )

        if ( download == True ):
            s = signals[i]
#            print s
            is_missing = True
            while ( s[0] == "download_progress" ):
                (name, op, pkg, ver, already, total) = s
                is_missing = False
                self.failIf( op != operation, "Wrong operation in download progress signal" )
                self.failIf( pkg != package, "Wrong package name in download progress signal" )
                if ( version != None and version != "" ):
                    self.failIf( ver != version, "Wrong version in download progress signal")
                i += 1
                if ( i >= len( signals ) ):
                    self.fail("No operation complete signal")
                    break
                s = signals[i]
            self.failIf( is_missing, "There is no download progress signal")
        else:
            for sig in signals:
                self.failIf( sig[0] == "download_progress", "There is a download progress signal even there shouldn't" )
                
        s = signals[i]
        is_missing = True
        if ( op_signal ):
            while ( s[0] == "operation_progress" ):
#                print s
                (name, op, pkg, ver, percentage) = s
                is_missing = False
                self.failIf( op != operation, "Wrong operation in operation progress signal")
                self.failIf( pkg != package, "Wrong package name in operation progress signal")
                if ( version != None and version != "" ):
                        self.failIf( ver != version, "Wrong version in operation progress signal")
                i += 1
                if ( i >= len( signals ) ):
                    self.fail("No operation complete signal")
                    break
                s = signals[i]
            self.failIf(is_missing, "There is no operation progress signal")
        else:
            self.failIf( s[0] == "operation_progress", "There's an operation progress signal even there shouldn't" )

        if ( pkg_list ):
            s = signals[i]
            self.failUnless( s[0] == "package_list_updated", "package_list_updated signal is missing" )
            i += 1

        s = signals[i]
        if ( many_complete == True ):
            is_missing = True
            while ( s[0] == "operation_complete" ):
#                print s
                (name, op, pkg, ver, err, reboot) = s
                if pkg != package:
                    i += 1
                    if (i < len(signals)):
                        s = signals[i]
                        continue
                is_missing = False
                self.failIf( op != operation, "Wrong operation in operation complete signal. Expected '%s', got '%s'" % (operation, op))
#                self.failIf( pkg != package, "Wrong package name in operation complete signal")
                if ( version != None and version != "" ):
                    self.failIf( ver != version, "Wrong version in operation complete signal")
                self.failIf( err != error, "Wrong error code in operation complete signal")
                i += 1
                if ( i >= len( signals ) ):
                    break
                s = signals[i]
            self.failIf( is_missing, "There is no operation complete signal")
        else:
            i += 1
            if ( s[0] != "operation_complete" ):
                self.fail("Operation complete signal is missing")
            else:
                (name, op, pkg, ver, err, reboot) = s
                self.failIf( op != operation, "Wrong operation in operation complete signal. Expected '%s', got '%s'" % (operation, op))
#                self.failIf( pkg != package, "Wrong package name in operation complete signal")
                if ( version != None and version != "" ):
                    self.failIf( ver != version, "Wrong version in operation complete signal")
                self.failIf( err != error, "Wrong error code in operation complete signal")

        l = len( signals )
#        print s
#        print i
#        print l

        self.failIf( l != i, "Wrong amount of signals!" )
#        s = signals[i]
#        self.failIf( s != None )
#        print signals[i]
#        print "check_signals_order() out"

